
#include <iostream.h>
#include <stdlib.h>
/* #include <strings.h> */
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>

#include "lifetime_polka.H"
#include "threadmap_polka.H"
#include "general.h"

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>

#define VERY_LARGE_INT  2000000000
typedef enum {BY_FILE, BY_SOCKET} input_type;

/* defined in Polka: polkawidgetmotif.C */
extern XtAppContext _context;
extern Display* _display;

extern void PolkaInit();
extern "C" {
extern void exit(int);
extern void  free(void*);
extern double atof(const char*);
extern void remove_input();
#include "dv_cminterf.h"
}

/* global variables */
int debug_flag = True;		// whether print debug info
#ifdef MEASURE
struct timeval start_time, end_time;
#endif

/* locally defined variables */

static int running_flag = True;	// whether views running or pause
int stepping_flag = 0;		// whether views stepping
static int speed_value = 0;		// speed of views

static int perturbation_threshold_us = VERY_LARGE_INT; // in micro-seconds
static int perturbation_threshold = VERY_LARGE_INT;    // in timestamp units
static unsigned long timestamp_resolution = 0;
static int    resize_const = False;    // resize policy
static int    number_of_thread_lines = 16; // initial no threads in window
static double h_zoom_ratio = 0.5;	// horizontal zoom ratio, from 0 to 1
static double v_zoom_ratio = 0.5;	// vertical zoom ratio, from 0 to 1
static double h_move_ratio = 0.5;      // horizontal move ratio, from 0 to 1
static double v_move_ratio = 0.5;      // vertical move ratio, from 0 to 1
static int    number_of_colors = 0;    // no. color names, if 0, dflt colors
static char   **names_of_colors = NULL;	// color names

static int num_monitor_exit = 0;
static DM_Type_thread_init*  thread_init_data = NULL;
static ThreadLifeTime_polka *lifetime_view = NULL;
static ThreadMap_polka *threadmap_view = NULL;

static
const char *usage = "usage: %s [-help] [-lines <number of threads in one screen>]\n\
        [-h_zoom_ratio <ratio 0 to 1>] [-v_zoom_ratio <ratio 0 to 1>]\n\
        [-h_move_ratio <ratio 0 to 1>] [-v_move_ratio <ratio 0 to 1>]\n\
        [-perturbation_threshold <microseconds>] \n\
        [-colors <number of colors> <color names>]\n\
	[-resize_constant]\n\
        [-isock_in <host name> <port number>]\n\
        [-usock_in <unix socket name>]\n\
        [-sock_in <unix socket name>]\n\
        [-file_in <file name>]\n";

/* forward definitions */

void QuitCB(Widget, XtPointer, XtPointer);
void PolkaRunPauseCB(Widget, XtPointer, XtPointer);
void PolkaStepCB(Widget, XtPointer, XtPointer);
void PolkaDelayCB(Widget, XtPointer, XtPointer);
void PolkaCloseCB(Widget, XtPointer, XtPointer);


void
ProcessArguments(int argc, char **argv)
{
    int i = 1, j;
    while (i < argc) {
	if (!strncmp(argv[i], "-help", 5)) {
	    fprintf(stderr, usage, argv[0]);
	    exit(0);
	} else if (!strncmp(argv[i], "-lines", 5)) {
	    number_of_thread_lines = atoi(argv[++i]);
	} else if (!strncmp(argv[i], "-h_zoom_ratio", 7)) {
	    h_zoom_ratio = atof(argv[++i]);
	} else if (!strncmp(argv[i], "-v_zoom_ratio", 7)) {
	    v_zoom_ratio = atof(argv[++i]);
	} else if (!strncmp(argv[i], "-h_move_ratio", 7)) {
	    h_move_ratio = atof(argv[++i]);
	} else if (!strncmp(argv[i], "-v_move_ratio", 7)) {
	    v_move_ratio = atof(argv[++i]);
	} else if (!strncmp(argv[i], "-resize_constant", 7)) {
	    resize_const = True;
	} else if (!strncmp(argv[i], "-perturbation_threshold", 5)) {
	    perturbation_threshold_us = atoi(argv[++i]);
	} else if (!strncmp(argv[i], "-colors", 6)) {
	    number_of_colors = atoi(argv[++i]);
	    if (number_of_colors < 0 || argc <= i+number_of_colors) {
		fprintf(stderr, usage, argv[0]);
		exit(0);
	    } else {
		names_of_colors =
		  (char**) malloc(sizeof(char*) * number_of_colors);
		for (j = 0; j < number_of_colors; j++) {
		    names_of_colors[j] = argv[++i];
		}
	    }
	} else {
	    fprintf(stderr, usage, argv[0]);
	}
	i++;
    }
}


/* handle trace events */
int
handle_threads_events(int event_id, void *data)
{

    if (thread_init_data != NULL &&
	event_id >= DM_THREAD_INIT && event_id <= DM_MONITOR_EXIT) {
	SimpleTraceDataRecordPtr  event = (SimpleTraceDataRecordPtr) data;
	event->type = event_id;  /* reassign type field */
printf("event_id: %d, event: <type=%d, P=%d, ts=%lu, thrd=%lx>\n", event_id,
       event->type, event->perturbation, event->timestamp, event->thread);
	if (event->perturbation >= perturbation_threshold) {
	    SimpleTraceDataRecord ptb_event;
	    ptb_event.type = SEVENT_PERTURBATION;
	    ptb_event.perturbation = event->perturbation;
	    ptb_event.timestamp =
		event->timestamp - event->perturbation;
	    ptb_event.thread = event->thread;
	    lifetime_view->update((char*) &ptb_event);
	    lifetime_view->draw();
	}
	lifetime_view->update((char*) event);
	threadmap_view->update((char*) event);
	lifetime_view->draw();
	threadmap_view->draw();
    }
    switch (event_id) {
    case SIMPLE_COMMAND:
    case FLOAT_COMMAND:
    case NAMED_COMMAND:
	printf("got some command ... igore it\n");
	break;

    case DM_THREAD_INIT:
	if (thread_init_data == NULL) {
	    DM_Type_thread_initPtr event = (DM_Type_thread_initPtr) data;
	    event->header.type = DM_THREAD_INIT;
	    thread_init_data = (DM_Type_thread_initPtr)
		malloc(sizeof(DM_Type_thread_init));
	    *thread_init_data = *event;
	    if (event->name) thread_init_data->name = strdup(event->name);
	    /* currently use old scheme */
	    DBG2("cthread_init: timestamp resolution %lu\n, num_lms %d\n",
		 thread_init_data->timestamp_resolution,
		 thread_init_data->num_lms);
	    timestamp_resolution = thread_init_data->timestamp_resolution;

	    lifetime_view->create((char*) event);
	    threadmap_view->create((char *) event);
	    lifetime_view->draw();
	    threadmap_view->draw();
	    if (perturbation_threshold_us < VERY_LARGE_INT) {
		perturbation_threshold = perturbation_threshold_us *
		    ((int) timestamp_resolution / 1000) / 1000;
	    } else {
		perturbation_threshold = VERY_LARGE_INT;
	    }

	}
	break;

    case DM_MONITOR_EXIT:
	if (thread_init_data != NULL) {
	    num_monitor_exit++;
	    if (num_monitor_exit >= thread_init_data->num_lms) {
		return True;
	    }
	}
    }
    return False;
}

extern int reader_init();
XtAppContext  app_context;
XtInputId in_id;
extern DEPort  in_dep;

main(int argc, char **argv)
{
    if (CMonInterfInit(&argc, argv) < 0) {
	fprintf(stderr, usage, argv[0]);
	exit(1);
    }
    ProcessArguments(argc, argv);

    RegisterEventHandler(-1, (EventHandlerProc) handle_threads_events);

    PolkaSetCallbacks(&PolkaRunPauseCB, &PolkaStepCB, &PolkaDelayCB,
		      &PolkaCloseCB);
    PolkaInit();
    app_context = _context;

    lifetime_view = new ThreadLifeTime_polka(number_of_thread_lines, 2.0,
					     h_zoom_ratio, v_zoom_ratio,
					     h_move_ratio, v_move_ratio,
					     resize_const,
					     number_of_colors,
					     names_of_colors);
    lifetime_view->create((char*) thread_init_data);
    lifetime_view->draw();
    threadmap_view = new ThreadMap_polka(number_of_thread_lines,
					 resize_const,
					 number_of_colors,
					 names_of_colors);
    threadmap_view->create((char*) thread_init_data);
    threadmap_view->draw();
    in_id = XtAppAddInput(app_context, 
			  file_id_IOfile(DEport_from_port(in_dep)), 
			  (XtPointer)XtInputReadMask,
			  (XtInputCallbackProc) ReadAndProcessInput, NULL);
    running_flag = True;
    XtAppMainLoop(app_context);
#ifdef MEASURE
	    if (num_lms <= 0) {
		int secs, usecs;
		gettimeofday(&end_time, NULL);
		secs = end_time.tv_sec - start_time.tv_sec;
		usecs = end_time.tv_usec - start_time.tv_usec;
		if (usecs < 0) {
		    secs--;
		    usecs += 1000000;
		}
		printf("start time <%d, %d>\n", start_time.tv_sec, start_time.tv_usec);
		printf("end time <%d, %d>\n", end_time.tv_sec, end_time.tv_usec);
		printf("execution time <%d, %d>\n", secs, usecs);
	    }
#endif
}



void
QuitCB(Widget, XtPointer, XtPointer)
{
    exit(0);
}


extern DEPort  in_dep;

void
remove_input()
{
    /* also called on eof */
    XtRemoveInput(in_id);
    running_flag = False;
    stepping_flag = 0;
}

void
PolkaRunPauseCB(Widget, XtPointer, XtPointer)
{
    if (running_flag) {
	remove_input();
    } else {
	in_id = XtAppAddInput(app_context, 
			      file_id_IOfile(DEport_from_port(in_dep)), 
			      (XtPointer)XtInputReadMask,
			      (XtInputCallbackProc) ReadAndProcessInput, NULL);
	running_flag = True;
	stepping_flag = 0;
#ifdef MEASURE
	gettimeofday(&start_time, NULL);
#endif
    }
}

void
PolkaDelayCB(Widget, XtPointer, XtPointer sl)
{

    XmScrollBarCallbackStruct *slider = (XmScrollBarCallbackStruct *)sl;

    speed_value = slider->value;
    fprintf(stderr, "speed value is %d\n", speed_value);
}

void
PolkaCloseCB(Widget, XtPointer, XtPointer )
{
    exit(0);
}

void
PolkaStepCB(Widget, XtPointer, XtPointer)
{
    /* running_flag = False; */
    stepping_flag++;
    ReadAndProcessInput();
}
