
#include <iostream.h>
#include "threadmap_polka.H"
#include "general.h"
extern int debug_flag;


typedef Text *TextPtr;

void
vZoomInCB(Widget, XtPointer client_data, XtPointer)
{
   View *v = (View *)client_data;
   double lx,by,rx,ty,changex,changey,zoomfactor = 0.5;

   v->GetCoord(lx,by,rx,ty);
   changey = ((ty-by) * (1.0-zoomfactor)) / 2.0;
   changex = 0.0;
   /* changex = ((rx-lx) * (1.0-zoomfactor)) / 2.0; */
   v->SetCoord(lx+changex, by+changey, rx-changex, ty-changey);
}


void
vZoomOutCB(Widget, XtPointer client_data, XtPointer)
{
   View *v = (View *)client_data;
   double lx,by,rx,ty,changex,changey,zoomfactor = 0.5;

   v->GetCoord(lx,by,rx,ty);
   changey = ((ty-by) * (1.0-zoomfactor)) / 2.0;
   changex = 0.0;
   /* changex = ((rx-lx) * (1.0-zoomfactor)) / 2.0; */
   v->SetCoord(lx-changex, by-changey, rx+changex, ty+changey);
}


ThreadMap_polka::ThreadMap_polka(int thrds_in_win,
				 int resize_const,
				 int number_of_colors,
				 char **names_of_colors)
	: ThreadMap()
{
    num_threads_in_unit_window = thrds_in_win;
    if (resize_const) {
	resize_mode = ConstantAspect;
    } else {
	resize_mode = CoordStretch;
    }
    global_time = 0;
    xsequ = 0.0;
    xthrdid = 0.1;
    xforkts = 0.25;
    xprocno = 0.4;
    xexitts = 0.5;
    xclr = 0.65;
    xname = 0.80;
    polkaview = NULL;
    origy = 0.1; yincr = (1.0 - origy) / num_threads_in_unit_window;
    thickness = yincr * 0.6;
    fontname = (char*)"-*-fixed-bold-r-*-*-13-100-*-*-*-*-*-*";
    color_depot = new Palette(number_of_colors, names_of_colors);
    max_num_threads = MAX_NUM_THREADS_INIT;
    name_texts = (Text **) new TextPtr[max_num_threads];
}

void
ThreadMap_polka::threadmapInitialize()
{
    Text  *text;
    Line  *line;
    double ytpos, ytpos2;

    if (polkaview == NULL) {
        polkaview = new View();
    }
    polkaview->SetPanCallbacks(0,0,0,0,vZoomInCB, vZoomOutCB);
    polkaview->Create("Thread Information View", resize_mode);
    
    // DBG1("timestamp_resolution is %u\n", timestamp_resolution);
    // text = new Text(polkaview, True, 0.5, 0.95, "black", "r24",
    //                "Threads Information", 1);
    // text->Originate(global_time);
    ytpos = 0.05; ytpos2 = 0.02;
    line = new Line(polkaview, True, 0.0, origy - 0.01, 1.0, 0.0,
		    (char*)"black", 0.5, 1.0, False);
    line->Originate(global_time);
    text = new Text(polkaview, True, xsequ, ytpos, (char*)"black", fontname,
		    "Sequ", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xsequ, ytpos2, (char*)"black", fontname,
		    "#", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xthrdid, ytpos, (char*)"black", fontname,
		    "Thread", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xprocno, ytpos, (char*)"black", fontname,
		    "Proc", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xprocno, ytpos2, (char*)"black", fontname,
		    "#", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xthrdid, ytpos2, (char*)"black", fontname,
		    "ID", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xforkts, ytpos, (char*)"black", fontname,
		    "Fork", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xforkts, ytpos2, (char*)"black", fontname,
		    "Time", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xexitts, ytpos, (char*)"black", fontname,
		    "Exit", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xexitts, ytpos2, (char*)"black", fontname,
		    "Time", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xclr, ytpos, (char*)"black", fontname,
		    "Color", 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xname, ytpos, (char*)"black", fontname,
		    "Thread Name", 0);
    text->Originate(global_time);
}


void
ThreadMap_polka::threadmapNewThread(int thread_idx,
				    unsigned long thread,
				    unsigned long timestamp,
				    int on_processor,
				    char *name)
{
    char *clr_name;
    char pbuf[20];
    double  ytpos, timeval;
    Text  *text;
    TextPtr *ntexts;
    Rectangle  *rect;
    int  i, oldmax;

    if (thread_idx >= max_num_threads) {
	oldmax = max_num_threads;
	max_num_threads += MAX_NUM_THREADS_INCR;
	ntexts = new TextPtr[max_num_threads];
	for (i = 0; i < oldmax; i++) {
	    ntexts[i] = name_texts[i];
	}
	delete(name_texts);
	name_texts = ntexts;
    }
    ytpos = origy + yincr * thread_idx;
    clr_name = color_depot->color(thread_idx);
    sprintf(pbuf, "%d", thread_idx);
    text = new Text(polkaview, True, xsequ, ytpos, clr_name, fontname,
		    pbuf, 0);
    text->Originate(global_time);
    sprintf(pbuf, "%X", (long)thread);
    text = new Text(polkaview, True, xthrdid, ytpos, clr_name, fontname,
		    pbuf, 0);
    text->Originate(global_time);
    sprintf(pbuf, "%d", on_processor);
    text = new Text(polkaview, True, xprocno, ytpos, clr_name, fontname,
		    pbuf, 0);
    text->Originate(global_time);
    timeval = (timestamp - init_timestamp) * timestamp_resolution_inv;
    sprintf(pbuf, "%.4f", timeval);
    text = new Text(polkaview, True, xforkts, ytpos, clr_name, fontname,
		    pbuf, 0);
    text->Originate(global_time);
    text = new Text(polkaview, True, xname, ytpos, clr_name, fontname,
		    name, 0);
    text->Originate(global_time);
    name_texts[thread_idx] = text;
    rect = new Rectangle(polkaview, True, xclr, ytpos, 0.14, thickness,
			 clr_name, True);
    rect->Originate(global_time);
    threadmapVScroll(thread_idx);
}


void
ThreadMap_polka::threadmapNewName(int thread_idx, char *name)
{
    char *clr_name;
    double  ytpos;
    Text  *text;
    
    if (name_texts[thread_idx]) {
	delete(name_texts[thread_idx]);
    }
    ytpos = origy + yincr * thread_idx;
    clr_name = color_depot->color(thread_idx);
    text = new Text(polkaview, True, xname, ytpos, clr_name, fontname,
		    name, 0);
    text->Originate(global_time);
    name_texts[thread_idx] = text;
}


void
ThreadMap_polka::threadmapThreadExit(int thread_idx,
				     unsigned long,
				     unsigned long timestamp)
{
    char *clr_name;
    char pbuf[20];
    double  ytpos, timeval;
    Text  *text;

    ytpos = origy + yincr * thread_idx;
    clr_name = color_depot->color(thread_idx);
    timeval = (timestamp - init_timestamp) * timestamp_resolution_inv;
    sprintf(pbuf, "%.4f", timeval);
    text = new Text(polkaview, True, xexitts, ytpos, clr_name, fontname,
		    pbuf, 0);
    text->Originate(global_time);
    threadmapVScroll(thread_idx);
}


void
ThreadMap_polka::threadmapVScroll(int thread_idx)
{
    double  x1, y1, x2, y2;
    double  ytpos, hlfheight;

    ytpos = origy + yincr * thread_idx;
    polkaview->GetCoord(x1, y1, x2, y2);
    if ((ytpos+thickness > y2) || (ytpos < y1)) {
	hlfheight = (y2 - y1)*0.5;
	if (ytpos < hlfheight) ytpos = hlfheight;
	polkaview->SetCoord(x1, ytpos - hlfheight, x2, ytpos + hlfheight);
    }
}
