#include "config.h"
#include <fcntl.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include <errno.h>
#include <sys/types.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <stdlib.h>
#include <time.h>
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_SOCKLIB_H
#include "sockLib.h"
#include "hostLib.h"
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#ifndef HAVE_WINSOCK_H
#ifdef HAVE_SYS_UN_H
#include <sys/un.h>
#endif
#include <netinet/in.h>
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#else
#include <winsock.h>
#endif
#ifdef HAVE_SYS_UIO_H
#include <sys/uio.h>
#endif
#include "assert.h"
#include "io.h"
#include "io_interface.h"
#include "io_internal.h"
#include "unix_defs.h"

#if !defined(SOCKET_DEFINED) && !defined(HAVE_WINDOWS_H)
extern int socket ARGS((int domain, int type, int protocol));
#endif
#if !defined(CONNECT_DEFINED) && !defined(HAVE_WINDOWS_H)
extern int connect ARGS((int s, struct sockaddr *name, int namelen));
#endif

static char test_byte = 'T';

static int
fill_hostaddr(void *addr, char *hostname)
{
#ifdef HAS_STRUCT_HOSTENT
    struct hostent *host_addr;
    
    if ((host_addr = gethostbyname(hostname)) == NULL) {
	return 0;
    }
    memcpy(addr, host_addr->h_addr, host_addr->h_length);
    return 1;
#else
    /* VxWorks ? */
    *((int *)addr) = hostGetByName(hostname);
    return 1;
#endif
}

extern int
establish_server_connection(iofile, do_fallback)
IOFile iofile;
int do_fallback;
{
    int sock;
    int junk_errno;
    char *junk_str;

    if (os_sockets_init_func) {
	 os_sockets_init_func();
    }
    if ((iofile->server_fd == (void*)-1) || 
	(os_server_write_func(iofile->server_fd, &test_byte, 1, &junk_errno,
			      &junk_str) != 1)) {
	/* reestablish connection, name_str is the machine name */
	struct sockaddr_in sock_addr;
	char *format_server_host = getenv("FORMAT_SERVER_HOST");

	if (format_server_host == NULL) {
	    format_server_host = FORMAT_SERVER_HOST;	/* from configure */
	}
	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	    fprintf(stderr, "Failed to create socket for PBIO format server connection.  Not enough File Descriptors?\n");
	    return 0;
	}
	
	sock_addr.sin_family = AF_INET;
		
	if (fill_hostaddr(&sock_addr.sin_addr, format_server_host) == 0) {
	    fprintf(stderr, "Unknown Host \"%s\" specified as PBIO format server.\n",
		    format_server_host);
	    return 0;
	}
	sock_addr.sin_port = htons(FS_PORT);

	if (connect(sock, (struct sockaddr *) &sock_addr,
		    sizeof sock_addr) < 0) {

	    if (!do_fallback) return 0;

	    /* fallback */
	    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		fprintf(stderr, "Failed to create socket for PBIO format server connection.  Not enough File Descriptors?\n");
	       return 0;
	    }
	    format_server_host = "marquesas.cc.gatech.edu";
	    sock_addr.sin_family = AF_INET;
	    if (fill_hostaddr(&sock_addr.sin_addr, format_server_host) == 0){
		fprintf(stderr, "Unknown Host \"%s\" specified as PBIO format server.\n",
			format_server_host);
		return 0;
	    }
	    sock_addr.sin_port = htons(FS_PORT);
	    if (connect(sock, (struct sockaddr *) &sock_addr,
			sizeof sock_addr) < 0) {
		fprintf(stderr, "Failed to connect to primary or fallback format servers.\n");
	        return 0;
	    }
	}
	
	iofile->server_fd = (void*)sock;
	server_write_header(iofile);
    }
    return 1;
}
	
extern int (*establish_server_connection_ptr)ARGS((IOFile iofile, int do_fallback));

extern
IOContext
create_IOcontext()
{
    IOFile iofile;

    /* fill in function pointer used in io_formats.c */
    establish_server_connection_ptr = establish_server_connection;

    iofile = new_IOFile();
    iofile->status = OpenNoHeader;
    iofile->IOversion = CURRENT_IO_VERSION;
    iofile->is_context = TRUE;
    reset_read_ahead(iofile);
    return (IOContext) iofile;
}

extern
IOContext
create_IOsubcontext(context)
IOContext context;
{
    IOFile iofile;

    /* fill in function pointer used in io_formats.c */
    establish_server_connection_ptr = establish_server_connection;

    iofile = new_IOFile();
    iofile->status = OpenNoHeader;
    iofile->IOversion = CURRENT_IO_VERSION;
    iofile->is_context = TRUE;
    iofile->master_context = context;
    reset_read_ahead(iofile);
    return (IOContext) iofile;
}
