#include <stdio.h>
#include <ctype.h>
#include "config.h"
#include "sensor.h"

extern char* sensorPrefixString;
extern char* TypeString(/* type */);
extern void  IOTypeString(/*type, type_name_str, type_name_iostr*/);
extern LocationP LocationInit(/* loc_type, loc_name, loc_lines */);
extern AttributeP AttributeInit(/* attr_type, attr_array_size, attr_name,
				   attr_expr */);
extern SensorP SensorInit(/* sensor_type, sensor_name, attrs, samp_attrs,
			     locs, pols */);

static char *sensorHFileHeader = "/***\n  ** This file is generated \
by the sensor specification compiler.\n  ** It contains sensor data structure definitions for a user specified \
sensor.\n  ***/\n\n\n";

static char *sensorCodeHeader = "/***\n  ** This file is generated \
by sensor compiler.\n  ** It contains sensor code to be inserted into \
the monitored\n  ** program.\n  ***/\n\n\n";

static char *eventFormatHeader = "/***\n  ** File: \"%s\"\n\
  ** This file is generated \
by sensor compiler.\n  ** It contains event format definitions to \
be linked to\n  ** the monitoring library.\n  ***/\n\n\n";

static char *iomacros= "#define IOOffset(p_type,field) \\\n\
((int) (((char *) (&(((p_type)NULL)->field))) - ((char *) NULL)))\n\
#if defined(__STDC__) || defined(__ANSI_CPP__)\n\
#define IOstr(s) #s\n\
#else\n\
#define IOstr(s) \"s\"\n\
#endif\n\
#define IOArrayDecl(type, size) IOstr(type[size])\n\
#define IOArrayDecl2(type, size, size2) IOstr(type[size][size2]])\n\n\
#ifndef NULL\n\
#define NULL 0\n\
#endif\n\n";


static char *iofielddef = "typedef struct _IOField {\n\
    char	*field_name;\n\
    char        *field_type;\n\
    int	        field_size;\n\
    int         field_offset;\n\
} IOField, *IOFieldList;\n\n";

/*** OutPolicy, InPolicy, FreePolicy ***/

void OutPolicy(policy, outf)
PolicyP  policy;
FILE*    outf;
{
    fprintf(outf, "%d\n", policy->type);
}

PolicyP InPolicy(inf)
FILE*    inf;
{
    PolicyP   policy;
    int       type;

    fscanf(inf, "%d", &type);
    PolicyInit(policy, (PolicyType)type);
    return policy;
}

void FreePolicy(policy)
PolicyP policy;
{
    if (policy) free(policy);
}


/*** OutLocation, InLocation, FreeLocation ***/

void OutLocation(location, outf)
LocationP  location;
FILE*      outf;
{
    int   i, numLine;
    
    fprintf(outf, "%d ", location->type);
    if (location->type == AT_LINES_LOCATION) {
	if (location->name) fprintf(outf, "%s ", location->name);
	else fprintf(outf, "a_null_file_name ");
	if (location->lines) {
	    numLine = *location->lines;
	    fprintf(outf, "%d ", numLine);
	    for (i = 1; i <= numLine; i++)
	      fprintf(outf, "%d ", location->lines[i]);
	} else fprintf(outf, "0 ");
    }
    
    fprintf(outf, "\n");
}

LocationP InLocation(inf)
FILE*    inf;
{
    LocationP  location;
    int        type, i, numLine;
    char       buf[256];

    fscanf(inf, "%d", &type);
    location = LocationInit(type, NULL, NULL);
    if (type == AT_LINES_LOCATION) {
	fscanf(inf, "%s", buf);
	if (strcmp(buf, "a_null_file_name")) {
	    StringInit(location->name, buf);
	} else location->name = NULL;

	fscanf(inf, "%d", &numLine);
	location->lines = (int*) malloc(sizeof(int)*(numLine+1));
	location->lines[0] = numLine;
	for (i = 1; i <= numLine; i++)
	  fscanf(inf, "%d", &location->lines[i]);
    }

    return location;
}

void FreeLocation(location)
LocationP  location;
{
    if (location) {
	if (location->name) free(location->name);
	if (location->lines) free(location->lines);
	free(location);
    }
}


/*** OutAttribute, InAttribute, FreeAttribute ***/

void OutAttribute(attribute, outf)
AttributeP  attribute;
FILE        *outf;
{
    if (attribute->name) fprintf(outf, "%s ", attribute->name);
    else fprintf(outf, "a_null_attribute_name ");
    fprintf(outf, "%d %d ", attribute->type, attribute->arraySize);

    if (attribute->expression)
      fprintf(outf, "%s", attribute->expression);
    else fprintf(outf, "a_null_expression");
    fprintf(outf, "\n");
}

AttributeP InAttribute(inf)
FILE       *inf;
{
    AttributeP  attribute;
    int         type, arraySize;
    char        buf[256];

    fscanf(inf, "%s", buf);
    fscanf(inf, "%d", &type);
    fscanf(inf, "%d", &arraySize);
    attribute = AttributeInit(type, arraySize, buf, NULL);
    if (!strcmp(attribute->name, "a_null_attribute_name")) {
	free(attribute->name);
	attribute->name = NULL;
    }
    fscanf(inf, "%s", buf);
    if (strcmp(buf, "a_null_expression")) {
	StringInit(attribute->expression, buf);
    } else attribute->expression = NULL;
    
    return attribute;
}

void FreeAttribute(attribute)
AttributeP  attribute;
{
    if (attribute) {
	if (attribute->name) free(attribute->name);
	if (attribute->expression) free(attribute->expression);
	free(attribute);
    }
}


/*** OutSensor, InSensor, FreeSensor ***/

void OutSensor(sensor, outf)
SensorP   sensor;
FILE        *outf;
{
    AttributeP   attr;
    LocationP    loc;
    PolicyP      pol;
    fprintf(outf, "%s ", (sensor->name)?sensor->name:"a_null_sensor_name ");
    fprintf(outf, "%d\n", sensor->sensorType);
    
    fprintf(outf, "%d\n", sensor->numAttributes);
    attr = sensor->attributes;
    while (attr) {
	OutAttribute(attr, outf);
	attr = attr->next;
    }
    if (sensor->sensorType == SAMPLE_SENSOR) {
	fprintf(outf, "%d\n", sensor->numSampAttributes);
	attr = sensor->samp_attrs;
	while(attr) {
	    OutAttribute(attr, outf);
	    attr = attr->next;
	}
    }

    fprintf(outf, "%d\n", sensor->numLocations);
    loc = sensor->locations;
    while (loc) {
	OutLocation(loc, outf);
	loc = loc->next;
    }

    fprintf(outf, "%d\n", sensor->numPolicies);
    pol = sensor->policies;
    while (pol) {
	OutPolicy(pol, outf);
	pol = pol->next;
    }
}

SensorP InSensor(inf)
FILE    *inf;
{
    SensorP  sensor;
    char     buf[256];
    int      type, i;

    AttributeP   attr, attr1;
    LocationP    loc,  loc1;
    PolicyP      pol,  pol1;

    fscanf(inf, "%s", buf);
    
    fscanf(inf, "%s", buf);
    fscanf(inf, "%d", &type);
    sensor = SensorInit(EVENT_SENSOR, buf, NULL, NULL, NULL, NULL);
    if (!strcmp(sensor->name, "a_null_sensor_name")) {
	free(sensor->name);
	sensor->name = NULL;
    }
    sensor->sensorType = type;

    fscanf(inf, "%d", &sensor->numAttributes);
    attr = attr1 = NULL;
    for (i = 0; i < sensor->numAttributes; i++) {
	if (!attr1) attr = attr1 = InAttribute(inf);
	else {
	    attr1->next = InAttribute(inf);
	    attr1->next->prev = attr1;
	    attr1 = attr1->next;
	}
    }
    sensor->attributes = attr;
    fscanf(inf, "%d", &sensor->numSampAttributes);
    attr = attr1 = NULL;
    for (i = 0; i < sensor->numSampAttributes; i++) {
	if (!attr1) attr = attr1 = InAttribute(inf);
	else {
	    attr1->next = InAttribute(inf);
	    attr1->next->prev = attr1;
	    attr1 = attr1->next;
	}
    }
    sensor->samp_attrs = attr;

    fscanf(inf, "%d", &sensor->numLocations);
    loc = loc1 = NULL;
    for (i = 0; i < sensor->numLocations; i++) {
	if (!loc1) loc = loc1 = InLocation(inf);
	else {
	    loc1->next = InLocation(inf);
	    loc1->next->prev = loc1;
	    loc1 = loc1->next;
	}
    }
    sensor->locations = loc;

    fscanf(inf, "%d", &sensor->numPolicies);
    pol = pol1 = NULL;
    for (i = 0; i < sensor->numPolicies; i++) {
	if (!pol1) pol = pol1 = InPolicy(inf);
	else {
	    pol1->next = InPolicy(inf);
	    pol1->next->prev = pol1;
	    pol1 = pol1->next;
	}
    }
    sensor->policies = pol;

    return sensor;
}

void FreeSensor(sensor)
SensorP  sensor;
{
    AttributeP   attr, attr1;
    LocationP    loc,  loc1;
    PolicyP      pol,  pol1;
    
    if (sensor) {
	if (sensor->name) free(sensor->name);
	attr = sensor->attributes;
	while (attr) {
	    attr1 = attr;
	    attr = attr->next;
	    FreeAttribute(attr1);
	}
	loc = sensor->locations;
	while (loc) {
	    loc1 = loc;
	    loc = loc->next;
	    FreeLocation(loc1);
	}
	pol = sensor->policies;
	while (pol) {
	    pol1 = pol;
	    pol = pol->next;
	    FreePolicy(pol1);
	}

	free(sensor);
    }
}


/*** OutSensorSpecification: to output sensor specification  (a list of
  ** sensors) to a file,
  ** InSensorSpecification: to input from such a file to internal data
  ** structure (create a list of sensors, and
  ** FreeSensorSpecification: to free memory occupied by the list of sensors.
  ***/

void OutSensorSpecification(numSensors, sensors, outf)
int       numSensors;
SensorP   sensors;
FILE      *outf;
{
    fprintf(outf, "%d\n", numSensors);
    while (sensors) {
	OutSensor(sensors, outf);
	sensors = sensors->next;
    }
}

SensorP InSensorSpecification(numSensors, inf)
int   *numSensors;
FILE  *inf;
{
    int i,   numSensor;
    SensorP  sensor, sensor1;

    fscanf(inf, "%d", &numSensor);
    sensor = sensor1 = NULL;
    for (i = 0; i < numSensor; i++) {
	if (!sensor1) sensor = sensor1 = InSensor(inf);
	else {
	    sensor1->next = InSensor(inf);
	    sensor1->next->prev = sensor1;
	    sensor1 = sensor1->next;
	}
    }

    *numSensors = numSensor;
    return sensor;
}

void FreeSensorSpecification(sensors)
SensorP sensors;
{
    SensorP p;

    while (sensors) {
	p = sensors;
	sensors = sensors->next;
	free(p);
    }
}



void GenerateFieldList(filep, sensorp)
FILE *filep;
SensorP sensorp;
{
    AttributeP  attr;
    char        *type_name_str, *type_name_iostr;

    fprintf(filep, "static IOField _Event_Format_sensor_%s_[] = {\n",
	    sensorp->name);
    attr = sensorp->attributes;
    while (attr) {
	fprintf(filep, "    {\"%s\", ", attr->name);
	IOTypeString(attr->type, &type_name_str, &type_name_iostr);
	if (attr->arraySize > 1) {
	    fprintf(filep, "\"%s[%d]\", sizeof(%s),\n",
		    type_name_iostr, attr->arraySize - 1,
		    type_name_str, attr->arraySize - 1);
	    fprintf(filep, "        IOOffset(Sensor_Type_%sPtr, %s[0])},\n",
		    sensorp->name, attr->name);
	} else {
	    fprintf(filep, "\"%s\", sizeof(%s),\n",
		    type_name_iostr, type_name_str);
	    fprintf(filep, "        IOOffset(Sensor_Type_%sPtr, %s)},\n",
		    sensorp->name, attr->name);
	}
	attr = attr->next;
    }
    if (sensorp->sensorType == SAMPLE_SENSOR) {
	attr = sensorp->samp_attrs;
	while (attr) {
	    fprintf(filep, "    {\"%s\", ", attr->name);
	    IOTypeString(attr->type, &type_name_str, &type_name_iostr);
	    if (attr->arraySize > 1) {
		fprintf(filep, "IOArrayDecl(%s, %d), sizeof(%s),\n",
			type_name_iostr, attr->arraySize - 1,
			type_name_str, attr->arraySize - 1);
		fprintf(filep, "        IOOffset(Sensor_Type_%sPtr, %s[0])},\n",
			sensorp->name, attr->name);
	    } else {
		fprintf(filep, "\"%s\", sizeof(%s),\n",
			type_name_iostr, type_name_str);
		fprintf(filep, "        IOOffset(Sensor_Type_%sPtr, %s)},\n",
			sensorp->name, attr->name);
	    }
	    attr = attr->next;
	}
    }
    fprintf(filep, "    {NULL, NULL, 0, 0}\n};\n\n");
}


/*** GenerateEventFormat: generate event format definitions (in PBIO format)
  ** from a list of sensors and write them into a file.  The file name of
  ** event format should be <root_name>.eventformat.c by default.
  ***/

void GenerateEventFormat(numSensors, sensors, startEventNumber,
			 efname, hfname)
SensorP sensors;
int     numSensors, startEventNumber;
char    *efname, *hfname;
{
    FILE 	*eff;
    SensorP	sensorp;

    /*** open event format file ***/
    if (!strcmp(efname, "-")) eff = stdout;
    else {
	eff  = fopen(efname, "w+");
	if (!eff) {
	    fprintf(stderr, "Cannot open %s\n", efname);
	    exit(1);
	}
    }

    fprintf(eff, eventFormatHeader, efname);
    fprintf(eff, "#include <stdio.h>\n\n");
    if (hfname) fprintf(eff, "#include \"%s\"\n\n", hfname);
    fprintf(eff, "%s", iomacros);
    fprintf(eff, "%s", iofielddef);

    sensorp = sensors;
    while (sensorp) {
	GenerateFieldList(eff, sensorp);
	sensorp = sensorp->next;
    }

    fprintf(eff, "int Number_Of_User_Specified_Events = %d;\n\n",
	    numSensors);
    fprintf(eff, "IOField *User_Specified_Sensor_Fields[] = {\n");
    sensorp = sensors;
    while (sensorp) {
	fprintf(eff, "    _Event_Format_sensor_%s_,\n", sensorp->name);
	sensorp = sensorp->next;
    }
    fprintf(eff, "    NULL,\n};\n\n");

    fprintf(eff, "int User_Specified_Sensor_Numbers[] = {\n");
    sensorp = sensors;
    while (sensorp) {
	fprintf(eff, "    Sensor_Number_%s,\n", sensorp->name);
	sensorp = sensorp->next;
    }
    fprintf(eff, "    0,\n};\n\n");

    fprintf(eff, "char *User_Specified_Sensor_Names[] = {\n");
    sensorp = sensors;
    while (sensorp) {
	fprintf(eff, "    \"%s\",\n", sensorp->name);
	sensorp = sensorp->next;
    }
    fprintf(eff, "    NULL,\n};\n\n");

    fclose(eff);
}


void
GenerateSensorHFile(numSensors, ssr_p, startEventNumber, shf, file_name)
SensorP ssr_p;
int     numSensors, startEventNumber;
FILE    *shf;
char    *file_name;
{
    char        sensor_num_str[256], sensor_type_str[256];
    AttributeP  attr;
    int		eventNumber = startEventNumber;
    char        buf[128];
    int         i, len, start;

    /*** generate header for .sensors.h file */
    fprintf(shf, "%s", sensorHFileHeader);

    buf[0] = '_';
    start = len = strlen(file_name);
    while ((start > 0) && (file_name[start-1] != '/')) start--;
    for (i = 0; i < len-start; i++) {
	if (file_name[start+i] == '.') buf[i+1] = '_';
	else if (islower(file_name[start+i])) {
	    buf[i+1] = toupper(file_name[start+i]);
	} else buf[i+1] = file_name[start+i];
    }
    buf[++i] = '_';
    buf[++i] = '\0';
    fprintf(shf, "#ifndef %s\n", buf);
    fprintf(shf, "#define %s\n\n", buf);

    while (ssr_p) {
	sprintf(sensor_num_str, "Sensor_Number_%s", ssr_p->name);
	sprintf(sensor_type_str, "Sensor_Type_%s", ssr_p->name);

	/*** head of a sensor .h information ***/
	fprintf(shf, "\n/*** sensor %s ***/\n\n", ssr_p->name);
	if (ssr_p->sensorType == STATE_SENSOR)
	  fprintf(shf, "#define %s  %d\n\n", sensor_num_str,
		eventNumber);
	else
	  fprintf(shf, "#define %s  %d\n\n", sensor_num_str, eventNumber);
	fprintf(shf, "typedef struct _%s {\n", sensor_type_str);

	/*** consider attributes of the sensor ***/
	attr = ssr_p->attributes;
	if (attr == NULL) {
	    fprintf(shf, "    int junk; /* some compiles don't like null structs */\n");
	}
	while (attr) {
	    fprintf(shf, "    %s %s", TypeString(attr->type), attr->name);
	    if (attr->arraySize > 1)
	      fprintf(shf, "[%d];\n", attr->arraySize - 1);
	    else fprintf(shf, ";\n");
	    attr = attr->next;
	}
	if (ssr_p->sensorType == SAMPLE_SENSOR) {
	    attr = ssr_p->samp_attrs;
	    while (attr) {
		fprintf(shf, "    %s %s", TypeString(attr->type), attr->name);
		if (attr->arraySize > 1)
		    fprintf(shf, "[%d];\n", attr->arraySize - 1);
		else fprintf(shf, ";\n");
		attr = attr->next;
	    }
	}
    
	/*** end of a sensor .h information ***/
	fprintf(shf, "} %s, *%sPtr;\n", sensor_type_str, sensor_type_str);

	if (ssr_p->sensorType == SAMPLE_SENSOR) {
	    fprintf(shf, "\ntypedef struct _ST_%s {\n", ssr_p->name);
	    attr = ssr_p->samp_attrs;
	    if (attr == NULL) {
		fprintf(shf, "    int junk; /* some compiles don't like null structs */\n");
	    }
	    while (attr) {
		fprintf(shf, "    %s %s", TypeString(attr->type), attr->name);
		if (attr->arraySize > 1)
		    fprintf(shf, "[%d];\n", attr->arraySize - 1);
		else fprintf(shf, ";\n");
		attr = attr->next;
	    }
	    fprintf(shf, "} ST_%s, *ST_%sPtr;\n",
		    ssr_p->name, ssr_p->name);
	}
	eventNumber++;
	fprintf(shf, "\n\n");
	ssr_p = ssr_p->next;
    }
    fprintf(shf, "#endif\n");
}


/*** GenerateOtherSensorCode: to generate sensor code of type other than
     sampling sensor ***/
static void
GenerateOtherSensorCode(scf, sensor_p)
    FILE     *scf;
    SensorP  sensor_p;
{
    char        sensor_descr_str[256], sensor_type_str[256];
    AttributeP  attr;

    sprintf(sensor_descr_str, "%s%s", "Sensor_Descriptor_", sensor_p->name);
    sprintf(sensor_type_str, "%s%s", "Sensor_Type_", sensor_p->name);

    fprintf(scf, "static sensor_descriptor %s = \n    {1, 0, 0, \"%s\", _Event_Format_sensor_%s_};\n",
	    sensor_descr_str, sensor_p->name, sensor_p->name);
    fprintf(scf, "int\n%s_%s(", sensorPrefixString, sensor_p->name);
    attr = sensor_p->attributes;
    while (attr) {
	fprintf(scf, "%s", attr->name);
	if (attr->next != NULL) fprintf(scf, ", ");
	attr = attr->next;
    }
    fprintf(scf, ")\n");
    attr = sensor_p->attributes;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf, "%s%s[%d];\n", TypeString(attr->type),
		    attr->name, attr->arraySize -1);
	} else {
	    fprintf(scf, "%s%s;\n", TypeString(attr->type), attr->name);
	}
	attr = attr->next;
    }
    fprintf(scf, "{\n");
    fprintf(scf, "    if (sensor_test_switch && (sensor_test_switch)(&%s)) {\n",
	    sensor_descr_str);
    if (sensor_p->have_array_fld) fprintf(scf, "        int  i;\n");
    fprintf(scf, "        int sensor_size = sensor_header_size + sizeof(%s);\n",
	    sensor_type_str);
    fprintf(scf, "        static char *aSensor = (char *)0;\n");
    fprintf(scf, "        %sPtr  theSpecificSensor; \n\n", sensor_type_str);
    fprintf(scf, "        if (aSensor == (char *) 0) aSensor = malloc(sensor_size);\n");
    fprintf(scf, "        if (sensor_init_header)\n             (sensor_init_header)(aSensor, &%s);\n", 
	    sensor_descr_str);
    fprintf(scf, "        theSpecificSensor = (%sPtr)(aSensor + sensor_header_size);\n", 
	    sensor_type_str);
    
    /*** consider attributes of the sensor ***/
    attr = sensor_p->attributes;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf, "        for (i = 0; i < %d; i++)\n            theSpecificSensor->%s[i] = %s[i];\n",
		    attr->arraySize - 1, attr->name, attr->name);
        } else {
	    fprintf(scf, "        theSpecificSensor->%s = %s;\n",
		    attr->name, attr->name);
	}
	attr = attr->next;
    }
    
    /*** end of a sensor code ***/

    fprintf(scf, "\n        while(!(sensor_non_blocking_submit_event)(");
    fprintf(scf, "aSensor, sensor_size, &%s, sensor_transport_info)) {\n", sensor_descr_str);
    fprintf(scf, "            if (sensor_update_header)\n");
    fprintf(scf, "              (sensor_update_header)(aSensor, %s);\n", sensor_descr_str);
    fprintf(scf, "        }\n    }\n");

    fprintf(scf, "    return 0;\n");
    fprintf(scf, "}\n\n");
}

/*** GenerateSampleSensorCode: to generate sampling sensor code ***/
static void
GenerateSampleSensorCode(scf, sensor_p)
    FILE     *scf;
    SensorP  sensor_p;
{
    char        sensor_num_str[256], sensor_type_str[256];
    AttributeP  attr;
    sprintf(sensor_num_str, "%s%s", "Sensor_Number_", sensor_p->name);
    sprintf(sensor_type_str, "%s%s", "Sensor_Type_", sensor_p->name);

    /*** generate Sample_<name> ***/
    fprintf(scf, "static int\nSample_%s(ss_id, event_buf)\n    int ss_id;\n    void* event_buf;\n{\n",
	    sensor_p->name);
    fprintf(scf, "    %sPtr  theSpecificSensor =\n        (%sPtr) event_buf+sensor_header_size;\n",
	    sensor_type_str, sensor_type_str);
    fprintf(scf, "    %sPtr  w_src =\n        (%sPtr) sensor_listof_ss_whole_fields[ss_id];\n",
	    sensor_type_str, sensor_type_str);
    fprintf(scf, "    ST_%sPtr  s_src =\n        (ST_%sPtr) sensor_listof_ss_sampled_fields[ss_id];\n",
	    sensor_p->name, sensor_p->name);
    if (sensor_p->have_array_fld) fprintf(scf, "    int  i;\n");
    fprintf(scf, "    (sensor_init_header)(event_buf, %s);\n", 
	    sensor_num_str);
    attr = sensor_p->attributes;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf, "    for (i = 0; i < %d; i++)\n        theSpecificSensor->%s[i] = w_src->%s[i];\n",
		    attr->arraySize - 1, attr->name, attr->name);
	} else {
	    fprintf(scf, "    theSpecificSensor->%s = w_src->%s;\n",attr->name, attr->name);
	}
	attr = attr->next;
    }
    attr = sensor_p->samp_attrs;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf, "    for (i = 0; i < %d; i++)\n        theSpecificSensor->%s[i] = s_src->%s[i];\n",
		    attr->arraySize - 1, attr->name, attr->name);
	} else {
	    fprintf(scf, "    theSpecificSensor->%s = s_src->%s;\n",attr->name, attr->name);
	}
	attr = attr->next;
    }
    fprintf(scf, "    return 0;\n}\n\n");

    /* generate RegisterSS_<name> ***/
    fprintf(scf, "void*\nRegisterSS_%s(samp_interval", sensor_p->name);
    attr = sensor_p->attributes;
    while (attr) {fprintf(scf, ", %s", attr->name); attr = attr->next; }
    attr = sensor_p->samp_attrs;
    while (attr) {fprintf(scf, ", %s", attr->name); attr = attr->next; }
    fprintf(scf, ")\nunsigned long  samp_interval;\n");
    attr = sensor_p->attributes;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf,"%s%s[%d];\n", TypeString(attr->type),
		    attr->name, attr->arraySize -1);
	} else {
	    fprintf(scf, "%s%s;\n", TypeString(attr->type), attr->name);
	}
	attr = attr->next;
    }
    attr = sensor_p->samp_attrs;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf,"%s%s[%d];\n", TypeString(attr->type),
		    attr->name, attr->arraySize -1);
	} else {
	    fprintf(scf, "%s%s;\n", TypeString(attr->type), attr->name);
	}
	attr = attr->next;
    }
    fprintf(scf, "{\n    %sPtr  w_event;\n    ST_%sPtr  s_event;\n    int  ss_num;\n",
	    sensor_type_str, sensor_p->name);
    if (sensor_p->have_array_fld) fprintf(scf, "    int  i;\n\n");
    /* fprintf(scf, "\n%s", sample_sensor_code_memalloc); */
    fprintf(scf, "    w_event = (%sPtr)(sensor_memory_alloc)(sizeof(%s));\n",
	    sensor_type_str, sensor_type_str);
    fprintf(scf, "    s_event = (ST_%sPtr)(sensor_memory_alloc)(sizeof(ST_%s));\n",
	    sensor_p->name, sensor_p->name);
    fprintf(scf, "    if (!w_event || !s_event) {\n");
    fprintf(scf, "        fprintf(stderr, \"cannot allocate memory for sampling sensors\\n\");\n");
    fprintf(scf, "        return (void*) -1;\n    }\n");
    fprintf(scf, "    if ((ss_num = (sensor_register_sampling_sensor)(samp_interval, Sample_%s)) < 0)\n        return (void*) -1;\n",
	    sensor_p->name);
    attr = sensor_p->attributes;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf,"    for (i = 0; i < %d; i++)\n        w_event->%s[i] = %s[i];\n",
		    attr->arraySize -1, attr->name, attr->name);
	} else {
	    fprintf(scf, "    w_event->%s = %s;\n", attr->name, attr->name);
	}
	attr = attr->next;
    }
    attr = sensor_p->samp_attrs;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf,"    for (i = 0; i < %d; i++)\n        s_event->%s[i] = %s[i];\n",
		    attr->arraySize -1, attr->name, attr->name);
	} else {
	    fprintf(scf, "    s_event->%s = %s;\n", attr->name, attr->name);
	}
	attr = attr->next;
    }
    fprintf(scf, "    sensor_listof_ss_whole_fields[ss_num] = w_event;\n");
    fprintf(scf, "    sensor_listof_ss_sampled_fields[ss_num] = s_event;\n");
    fprintf(scf, "    (sensor_turn_on_sampling_sensor)(ss_num);\n    return (void*) ss_num;\n}\n\n");
    /* generate sampling sensor call */
    fprintf(scf, "void\nSS_%s(ss_num", sensor_p->name);
    attr = sensor_p->samp_attrs;
    while (attr) {fprintf(scf, ", %s", attr->name); attr = attr->next; }
    fprintf(scf, ")\nvoid*          ss_num;\n");
    attr = sensor_p->samp_attrs;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf,"%s%s[%d];\n", TypeString(attr->type),
		    attr->name, attr->arraySize -1);
	} else {
	    fprintf(scf, "%s%s;\n", TypeString(attr->type), attr->name);
	}
	attr = attr->next;
    }
    fprintf(scf, "{\n    ST_%sPtr  s_event =\n        (ST_%sPtr) sensor_listof_ss_sampled_fields[(int)ss_num];\n",
	    sensor_p->name, sensor_p->name);
    attr = sensor_p->samp_attrs;
    while (attr) {
	if (attr->arraySize > 1) {
	    fprintf(scf,"    for (i = 0; i < %d; i++)\n        s_event->%s[i] += %s[i];\n",
		    attr->arraySize -1, attr->name, attr->name);
	} else {
	    fprintf(scf, "    s_event->%s += %s;\n", attr->name, attr->name);
	}
	attr = attr->next;
    }
    fprintf(scf, "}\n\n");
}

/*** GenerateSensorCodeFile: generate sensor code from a list of sensors ***/

static char* sensor_code_extern_glb =
"/*** global variables ***/\n\
extern int (*sensor_non_blocking_submit_event)(/* buffer, size */);\n\
extern void (*sensor_turn_on_sampling_sensor)(/* ss_num */);\n\
extern int (*sensor_register_sampling_sensor)(/* interval, sample_proc */);\n\
extern int (*sensor_test_switch)(/*ss_num */);\n\
extern void (*sensor_init_header)(/*buffer, ss_num */);\n\
extern void (*sensor_update_header)(/*buffer*/);\n\
extern char *(*sensor_memory_alloc)(/*size*/);\n\
extern void *sensor_transport_info; /* transport specific info */\n\
extern int sensor_header_size; /* transport specific header size */\n\
extern void** sensor_listof_ss_whole_fields;\n\
extern void** sensor_listof_ss_sampled_fields;\n\
extern char *malloc(/*size*/);\n\n\n";

static char *sensor_code_extern_glb2 = 
"extern void (*sensor_memory_free)(/*PtrToData*/);\n\n";

static char*sensordef =
"typedef struct {\n\
    int struct_version;\n\
    int sensor_initialized;\n\
    int sensor_enabled;\n\
    char *sensor_name;\n\
    IOFieldList sensor_field_list;\n\
    int sensor_type_value;\n\
} sensor_descriptor;\n\n";

void
GenerateSensorCodeFile(numSensors, sensors, startEventNumber, scf, shf_name)
SensorP sensors;
int     numSensors, startEventNumber;
FILE    *scf;
char    *shf_name;
{
    /*** generate header for .sensorcode.c file */
    fprintf(scf, "%s", sensorCodeHeader);

    fprintf(scf, "#include \"%s\"\n\n\n", shf_name);

    fprintf(scf, "%s", sensor_code_extern_glb);

    fprintf(scf, "%s", iomacros);
    fprintf(scf, "%s", iofielddef);
    fprintf(scf, "%s", sensordef);

    while (sensors) {
	if (sensors->sensorType == SAMPLE_SENSOR) {
	    fprintf(scf, "/*** sampling sensor %s ***/\n", sensors->name);
	    GenerateFieldList(scf, sensors);
	    GenerateSampleSensorCode(scf, sensors);
	} else {
	    fprintf(scf, "/*** sensor %s ***/\n", sensors->name);
	    GenerateFieldList(scf, sensors);
	    GenerateOtherSensorCode(scf, sensors);
	}
	sensors = sensors->next;
    }
}


void
GenerateSensorExtHFile(numSensors, sensors, startEventNumber, sehf)
SensorP sensors;
int     numSensors, startEventNumber;
FILE    *sehf;
{
    AttributeP  attr;

    while (sensors) {
	if (sensors->sensorType == SAMPLE_SENSOR) {
	    fprintf(sehf, "extern void*\nRegisterSS_%s( /*\nunsigned long  samp_interval;\n",
		    sensors->name);
	    attr = sensors->attributes;
	    while (attr) {
		if (attr->arraySize > 1) {
		    fprintf(sehf, "%s%s[%d],\n", TypeString(attr->type),
			    attr->name, attr->arraySize - 1);
		} else {
		    fprintf(sehf, "%s%s,\n", TypeString(attr->type), attr->name);
		}
		attr = attr->next;
	    }
	    attr = sensors->samp_attrs;
	    while (attr) {
		if (attr->arraySize > 1) {
		    fprintf(sehf, "%s%s[%d],\n", TypeString(attr->type),
			    attr->name, attr->arraySize - 1);
		} else {
		    fprintf(sehf, "%s%s,\n", TypeString(attr->type), attr->name);
		}
		attr = attr->next;
	    }
	    fprintf(sehf, "*/);\n\n");
	    fprintf(sehf, "extern void\nSS_%s( /*\nvoid*          ss_num;\n",
		    sensors->name);
	    attr = sensors->samp_attrs;
	    while (attr) {
		if (attr->arraySize > 1) {
		    fprintf(sehf, "%s%s[%d],\n", TypeString(attr->type),
			    attr->name, attr->arraySize - 1);
		} else {
		    fprintf(sehf, "%s%s,\n", TypeString(attr->type), attr->name);
		}
		attr = attr->next;
	    }
	    fprintf(sehf, "*/);\n\n");
	} else {
	    fprintf(sehf, "extern int\n%s_%s( /*\n", sensorPrefixString, 
		    sensors->name);
	    attr = sensors->attributes;
	    while (attr) {
		if (attr->arraySize > 1) {
		    fprintf(sehf, "%s%s[%d],\n", TypeString(attr->type),
			    attr->name, attr->arraySize - 1);
		} else {
		    fprintf(sehf, "%s%s,\n", TypeString(attr->type), attr->name);
		}
		attr = attr->next;
	    }
	    fprintf(sehf, "*/);\n\n");
	}
	sensors = sensors->next;
    }
}

