#include <stdio.h>
#include <stdlib.h>

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/tcp.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <fcntl.h>

#include "NetComm.h"

char *NetComm::getName() {
  return name;
}

NetComm::NetComm() {
  strcpy(name, "NetComm");
  strcpy(bufferedMessage, NETCOMM_NOMESSAGE);

  strcpy(host, DEFAULT_NETCOMM_HOST);
  port = DEFAULT_NETCOMM_PORT;
  the_socket = -1;
  server_socket = -1;
  comm_type = DEFAULT_NETCOMM_TYPE;
  debug = 0;
}

NetComm::~NetComm() {
  if (the_socket > 0)
    close(the_socket);

  if (server_socket > 0)
    close(server_socket);
}

void NetComm::readData() {

}

void NetComm::setPort(int port) {
  this->port = port;
}

void NetComm::setHost(char *host) {
  strcpy(this->host, host);
}

void NetComm::setCommType(int type) {
  this->comm_type = type;
}

void NetComm::init() {
  bool ret = false;

  if (debug > 1) printf("%s: Attempting to connect to host %s on port %d:", getName(), host, port);
  while(ret == false) {
    if (debug > 1) printf(".");
    ret = connect_();
  }
  if (debug > 1) printf("\n");
}


void NetComm::sendMessage(char *message) {
    char buffer[DEFAULT_NETCOMM_BUFFER_SIZE];
    strcpy(buffer, message);

    write_(message);
}

bool NetComm::connect_(bool read_init_msg) {
	char buffer[DEFAULT_NETCOMM_BUFFER_SIZE];
	struct sockaddr_in addr, cli_addr;
	struct hostent *h = gethostbyname(host);
	int clilen, n;
        if (debug) printf("CONNECT_\n");
	if (!h) {
		fprintf(stderr, "Error: (%s): Could not resolve the NetComm host: %s\n", getName(), host);
		return false;
	}

    if (debug) printf("Connecting to host %s on port %d (Type=%d)\n", host, port, comm_type);

	memset(&addr, 0, sizeof(struct sockaddr_in));
	memcpy(&addr.sin_addr, h->h_addr, h->h_length);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);

    if (comm_type == NETCOMM_TYPE_CLIENT) {
      the_socket = socket(AF_INET, SOCK_STREAM, 0);
	  if (the_socket < 0) {
		fprintf(stderr, "Error: (%s): Couldn't create a new socket\n", getName());
		return false;
	  }

	  if (connect(the_socket, (struct sockaddr*)&addr, sizeof(addr)) < 0) {
	  	  fprintf(stderr, "Error: %s: error connecting\n", getName());
		  close(the_socket);
		  return false;
	  }

	  //write_("InitReq\r\n");
	  //read_(buffer);
	  if (debug > 1) printf("Read: %s\n", buffer);

	  // read the initial message from the server
  	  if (read_init_msg) {
		ssize_t ret = read(the_socket, buffer, sizeof(buffer));
		if (ret <= 0) {
			fprintf(stderr, "Error: %s: Error reading the initial message from the NetComm\n", getName());
			close(the_socket);
			return false;
		}

		fwrite(buffer, ret, 1, stdout);
	  }
	} else {
      server_socket = socket(AF_INET, SOCK_STREAM, 0);
	  if (server_socket < 0) {
		fprintf(stderr, "Error: %s: couldn't create a new socket\n", getName());
		return false;
	  }

	  if (bind(server_socket, (struct sockaddr *) &addr,
            sizeof(addr)) < 0) {
              fprintf(stderr, "%s: Error: Could not bind\n", getName());
              return false;
            }
        listen(server_socket,5);
        clilen = sizeof(cli_addr);
        the_socket = accept(server_socket,
                 (struct sockaddr *) &cli_addr,
                 (socklen_t *) (&clilen));
        if (the_socket < 0) {
          fprintf(stderr, "%s: Error: Could not accept\n", getName());
          return false;
        }
        //read_(buffer);
        printf("Init message from server: %s\n",buffer);
        //n = write_("InitAck\r\n");
        if (n == false) {
            fprintf(stderr, "%s: Error reading from socket for init message\n", getName());
            return false;
        }

	}

    // Set socket to be non-blocking
    int flags;
    if ((flags = fcntl(the_socket, F_GETFL, 0)) == -1)
        flags = 0;
    fcntl(the_socket, F_SETFL, flags | O_NONBLOCK);

	return true;
}

bool NetComm::write_(char *message) {
    int size = strlen(message)+1;

    if (size > DEFAULT_NETCOMM_BUFFER_SIZE) {
      fprintf(stderr, "%s: Warning: Message is longer than buffer size (proper buffering not implemented yet)\n", getName());
      return false;
    }

    // TODO Split into buffers!
    if (debug) printf("SEND %d bytes\n", DEFAULT_NETCOMM_BUFFER_SIZE);
    if (send(the_socket, message, DEFAULT_NETCOMM_BUFFER_SIZE, 0) < 0) {
		fprintf(stderr, "%s: Error writing to the socket", getName());
      return false;
	}

	return true;
}

bool NetComm::read_(char *message) {
	char buffer[DEFAULT_NETCOMM_BUFFER_SIZE];
	int ret = -1;


    if (debug) printf("READING\n");

    if (debug) printf("RECV %d bytes\n", DEFAULT_NETCOMM_BUFFER_SIZE);
    ret = recv(the_socket, buffer, DEFAULT_NETCOMM_BUFFER_SIZE, 0);
    if (ret < 0) {
      if (debug) printf("NO DATA\n");
      return false;
    }

    if (debug) printf("RECEIVED MESSAGE: %s\n", buffer);

    strcpy(message, buffer);
    return true;
}
