/**********************************************************************
 **                                                                  **
 **                          HServerTypes.h                          **
 **                                                                  **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2004 - 2008, Georgia Tech Research Corporation        **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: HServerTypes.h,v 1.6 2010-05-03 22:10:07 zkira Exp $ */

#ifndef HSERVER_TYPES_H
#define HSERVER_TYPES_H

#include <string>

using std::string;

static const double HS_EPSILON  = 0.000001;

static const int HS_POSE_EXTRADATA_SIZE  = 1024;

typedef struct HSPoseExtra_t {
    double time;
    char data[HS_POSE_EXTRADATA_SIZE];
};

typedef struct HSLocation_t {
    double x;
    double y;
    double z;
    HSPoseExtra_t extra;
};

typedef struct HSRotation_t {
    double yaw;
    double pitch;
    double roll;
    HSPoseExtra_t extra;
};

typedef struct HSPose_t {
    HSLocation_t loc;
    HSRotation_t rot;
    HSPoseExtra_t extra;
};

typedef HSLocation_t HSTransVelocity_t;

typedef HSRotation_t HSAngularVelocity_t;

typedef struct HSVelocityExtra_t {
    double time;
    bool backward;
    char data[HS_POSE_EXTRADATA_SIZE];
};

typedef struct HSVelocity_t {
    HSTransVelocity_t transVel;
    HSAngularVelocity_t angVel;
    HSVelocityExtra_t extra;
};

typedef struct HSSpeedExtra_t {
    double time;
    char data[HS_POSE_EXTRADATA_SIZE];
};

typedef struct HSAltitudeSpeed_t {
    double value;
    HSSpeedExtra_t extra;
};

typedef struct HSDriveSpeed_t {
    double value;
    HSSpeedExtra_t extra;
};

typedef struct HSSteerSpeed_t {
    double value;
    HSSpeedExtra_t extra;
};

typedef struct HSArmSpeed_t {
    double value;
    HSSpeedExtra_t extra;
};

typedef struct HSSpeed_t {
    HSDriveSpeed_t driveSpeed;
    HSSteerSpeed_t steerSpeed;
    HSAltitudeSpeed_t altitudeSpeed;
    HSArmSpeed_t armSpeed;
    HSSpeedExtra_t extra;
};

const string HS_MODULE_NAME_APM = "Apm";
const string HS_MODULE_NAME_CAMERA = "Camera";
const string HS_MODULE_NAME_COGNACHROME = "Cognachrome";
const string HS_MODULE_NAME_COMPASS_3DMG = "Compass3DMG";
const string HS_MODULE_NAME_COMPASS_KVHC100 = "CompassKVHC100";
const string HS_MODULE_NAME_GPS = "Gps";
const string HS_MODULE_NAME_CRICKET = "Cricket";
const string HS_MODULE_NAME_GYRO_DMUVGX = "GyroDMUVGX";
const string HS_MODULE_NAME_IPT = "IPT";
const string HS_MODULE_NAME_JBOX = "JBox";
const string HS_MODULE_NAME_LASER_SICK = "LaserSICK";
const string HS_MODULE_NAME_LASER_GAZEBO = "GazeboLaser";
const string HS_MODULE_NAME_LASER_USARSIM = "USARSimLaser";
const string HS_MODULE_NAME_OPENCV = "OpenCV";
const string HS_MODULE_NAME_RCOM = "RCom";
const string HS_MODULE_NAME_ROBOT_AMIGOBOT = "Amigobot";
const string HS_MODULE_NAME_ROBOT_ATRVJR = "ATRVJr";
const string HS_MODULE_NAME_ROBOT_FRED = "Fred";
const string HS_MODULE_NAME_ROBOT_GAZEBO = "GazeboRobot";
const string HS_MODULE_NAME_ROBOT_NOMAD = "Nomad";
const string HS_MODULE_NAME_ROBOT_PIONEER = "Pioneer";
const string HS_MODULE_NAME_ROBOT_PIONEER2 = "Pioneer2";
const string HS_MODULE_NAME_ROBOT_POSECALC = "PoseCalc";
const string HS_MODULE_NAME_ROBOT_URBAN = "Urban";
const string HS_MODULE_NAME_ROBOT_USARSIM = "USARSimRobot";
const string HS_MODULE_NAME_ROBOT_ROVIO = "RovioRobot";
const string HS_MODULE_NAME_ROBOT_SKYBOTIX = "SkybotixRobot";
const string HS_MODULE_NAME_VIDEO_FRAMEGRABBER = "FrameGrabber";
const string HS_MODULE_NAME_VIDEO_GAZEBO = "GazeboVideo";
const string HS_MODULE_NAME_VIDEO_USARSIM = "USARSimVideo";
const string HS_MODULE_NAME_VIDEO_ROVIO = "RovioVideo";
const string HS_MODULE_NAME_XWINDOW = "Xwindow";

typedef enum HSRobotTypes_t {
    HS_ROBOT_TYPE_AMIGOBOT,
    HS_ROBOT_TYPE_ATRVJR,
    HS_ROBOT_TYPE_FRED,
    HS_ROBOT_TYPE_GAZEBO,
    HS_ROBOT_TYPE_NOMAD,
    HS_ROBOT_TYPE_PIONEER,
    HS_ROBOT_TYPE_PIONEER2,
    HS_ROBOT_TYPE_URBAN,
    HS_ROBOT_TYPE_USARSIM,
    HS_ROBOT_TYPE_ROVIO,
    HS_ROBOT_TYPE_SKYBOTIX
};

typedef enum HSVideoTypes_t {
    HS_VIDEO_TYPE_FRAMEGRABBER,
    HS_VIDEO_TYPE_GAZEBO,
    HS_VIDEO_TYPE_USARSIM,
    HS_VIDEO_TYPE_ROVIO
};

typedef enum HSGazeboTypes_t {
    HS_GAZEBO_TYPE_ROBOT,
    HS_GAZEBO_TYPE_LASER,
    HS_GAZEBO_TYPE_VIDEO
};

typedef enum HSUSARSimTypes_t {
    HS_USARSIM_TYPE_ROBOT,
    HS_USARSIM_TYPE_LASER,
    HS_USARSIM_TYPE_VIDEO
};

typedef enum HSRovioTypes_t {
    HS_ROVIO_TYPE_ROBOT,
    HS_ROVIO_TYPE_VIDEO
};

typedef enum HSSkybotixTypes_t {
    HS_SKYBOTIX_TYPE_ROBOT,
    HS_SKYBOTIX_TYPE_VIDEO
};

typedef struct HSDateString_t {
    string year;
    string month;
    string day;
    string hour;
    string min;
    string sec;
};

#endif

/**********************************************************************
# $Log: HServerTypes.h,v $
# Revision 1.6  2010-05-03 22:10:07  zkira
# Cricket sensor, blob tracking, rovio (inc. IR sensor), skybotix interface (unfinished)
#
# Revision 1.4  2009/03/02 18:53:21  zkira
# Added Rovio
#
# Revision 1.3  2008/09/09 20:24:55  zkira
# Crude altitude control
#
# Revision 1.2  2008/07/15 22:33:47  endo
# OpenCV, Gazebo and USARSim added.
#
# Revision 1.11  2008/02/01 23:18:24  endo
# USARSimLaser added.
#
# Revision 1.10  2008/01/21 19:14:15  endo
# Regular backup.
#
# Revision 1.9  2008/01/09 22:54:03  endo
# USARSimRobot added.
#
# Revision 1.8  2006/11/21 23:13:38  endo
# FrameGrabber separated from Video class.
#
# Revision 1.7  2006/11/20 22:55:51  endo
# OpenCV added.
#
# Revision 1.6  2006/11/17 23:35:53  endo
# GazeboVideo implemented.
#
# Revision 1.5  2006/08/14 21:12:13  endo
# RComHandler implemented.
#
# Revision 1.4  2006/08/08 21:33:49  endo
# GazeboLaser added.
#
# Revision 1.3  2006/07/31 22:01:14  endo
# Gazebo -> GazeboRobot
#
# Revision 1.2  2006/07/21 23:35:01  endo
# Gazebo integrated into HServer
#
# Revision 1.1.1.1  2006/07/21 17:42:08  endo
# HServer for Gazebo (based on MissionLab-7.0.20060712.tar.gz)
#
# Revision 1.1.1.1  2006/07/12 13:37:56  endo
# MissionLab 7.0
#
# Revision 1.1  2006/06/05 21:35:51  endo
# PoseCalculator from MARA 2020 migrated
#
# Revision 1.1  2004/09/10 19:43:26  endo
# New PoseCalc integrated.
#
#**********************************************************************/
