/**********************************************************************
 **                                                                  **
 **                         RComAPIFactory.h                         **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  A factor class for RComAPI.                                     **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef RComAPIFactory_H
#define RComAPIFactory_H

/* $Id: RComAPIFactory.h,v 1.1 2008-07-15 22:33:57 endo Exp $ */

#include <string>
#include "RComAPI.h"
#include "RComClient.h"
#include "RComServer.h"
#include "RComInterthread.h"

using std::string;

typedef enum RComAPITypes_t {
    RCOMAPI_TYPE_RCOM_CLIENT,
    RCOMAPI_TYPE_RCOM_SERVER,
    RCOMAPI_TYPE_RCOM_INTERTHREAD,
    NUM_RCOMAPI_TYPES
};

//----------------------------------------------------------------
// Sole purpose of this class to instantiate the implementation
// of RComAPI.
//----------------------------------------------------------------
class RComAPIFactory {

public:
    static RComAPI *createRComAPI(int rcomAPIType);
};

inline RComAPI *RComAPIFactory::createRComAPI(int rcomAPIType)
{
    switch (rcomAPIType) {

        case RCOMAPI_TYPE_RCOM_CLIENT:
            return (new RComClient());

        case RCOMAPI_TYPE_RCOM_SERVER:
            return (new RComServer());

        case RCOMAPI_TYPE_RCOM_INTERTHREAD:
            return (new RComInterthread());
    }

    return NULL;
}

#endif
/**********************************************************************
 * $Log: RComAPIFactory.h,v $
 * Revision 1.1  2008-07-15 22:33:57  endo
 * OpenCV, Gazebo and USARSim added.
 *
 * Revision 1.2  2006/08/09 21:46:53  endo
 * -ansi -pedantic added to Makefile
 *
 * Revision 1.1  2006/08/09 21:12:37  endo
 * RCom added.
 *
 * Revision 1.3  2006/02/24 11:50:33  endo
 * RComm_Controller checked in.
 *
 * Revision 1.2  2006/02/22 07:17:22  endo
 * RComServer and RComClient implemented.
 *
 * Revision 1.1  2006/02/20 14:14:48  endo
 * RComAPI checked.
 *
 **********************************************************************/
