/**********************************************************************
 **                                                                  **
 **                           RComClient.h                           **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  This is a header file for RComClient.cc.                        **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef RComClient_H
#define RComClient_H

/* $Id: RComClient.h,v 1.1 2008-07-15 22:33:59 endo Exp $ */

#include <pthread.h>
#include <time.h>
#include <string>
#include <vector>

#include "RComAPI.h"

using std::string;
using std::vector;

class RComClient : public RComAPI {

    typedef enum InitiConnectionStatus_t {
        INIT_CONNECTION_STATUS_FAILURE,
        INIT_CONNECTION_STATUS_NOT_READY,
        INIT_CONNECTION_STATUS_SUCCESS
    };

    typedef struct ServerData_t
    {
        int recipientID;
        char *data;
        int dataSize;
        bool enfoceReliable;
    };

protected:
    pthread_t readerThread_;
    string serverHostMachine_;
    int shSocketFd_;
    int serverPort_;
    int myID_;
    bool readerThreadIsUp_;
    void *extraCallbackFuncArg_;

    void (*readerCallbackFunc_)(int, const void *, size_t, bool, void *);

    static pthread_mutex_t socketMutex_;

    static const struct timespec READER_SLEEP_TIME_NSEC_;
    static const string EMPTY_STRING_;
    static const int INVALID_SOCKET_FD_;

    ServerData_t readServerData_(void);
    int pollServer_(void);
    int readInteger_(const int fd);
    int connectToServer_(string hostMacine, int port);
    bool readNBytes_(const int fd, unsigned char *buf, const int n);
    void readerMainLoop_(void);
    void sendCommandToServer_(const unsigned char cmd);
    void writeInteger_(const int fd, int value);
    void processServerData_(ServerData_t *serverData);

    static void *startReaderThread_(void *data);

public:
    RComClient(void);
    ~RComClient(void);
    bool initialize(
        int myID,
        void (*readerCallbackFunc)(int, const void *, size_t, bool, void *),
        void *extraCallbackFuncArg);
    bool sendData(
        int recipientID,
        const void *data,
        size_t dataSize,
        bool enfoceReliable);
    void setServerPort(int port);
};

inline void RComClient::setServerPort(int port)
{
    serverPort_ = port;
}

#endif
/**********************************************************************
 * $Log: RComClient.h,v $
 * Revision 1.1  2008-07-15 22:33:59  endo
 * OpenCV, Gazebo and USARSim added.
 *
 * Revision 1.2  2006/08/09 21:46:53  endo
 * -ansi -pedantic added to Makefile
 *
 * Revision 1.1  2006/08/09 21:12:37  endo
 * RCom added.
 *
 * Revision 1.5  2006/02/25 22:10:23  endo
 * The user can now specify ports to be used.
 *
 * Revision 1.4  2006/02/25 16:17:49  endo
 * rcomserver added
 *
 * Revision 1.3  2006/02/24 11:50:34  endo
 * RComm_Controller checked in.
 *
 * Revision 1.2  2006/02/22 07:17:22  endo
 * RComServer and RComClient implemented.
 *
 * Revision 1.1  2006/02/20 14:23:55  endo
 * RComAPI checked in.
 *
 **********************************************************************/
