/**********************************************************************
 **                                                                  **
 **                        RComInterthread.h                         **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef RComINTERTHREAD_H
#define RComINTERTHREAD_H

/* $Id: RComInterthread.h,v 1.1 2008-07-15 22:33:59 endo Exp $ */

#include <string>
#include <vector>

#include "RComDefaultSetups.h"
#include "RComAPI.h"
#include "RComm_Controller.h"

using std::string;
using std::vector;

class RComInterthread : public RComAPI {

protected:
	RComm_Controller *rcommController_;
    char peerRComIPs_[RCOM_NUM_PATHS][IP_ARRAY_SIZE];
    unsigned char localReadDataBuffer_[RCOM_MAX_PAYLOAD_SIZE];
    int myID_;
    int interRComPort_;
    void *extraCallbackFuncArg_;
    bool controllerInitialized_;

    void (*applicationCallbackFunc_)(int, const void *, size_t, bool, void *);

    static void controllerCallback_(
        int recipientID,
        int mybufferlength,
        bool reliabilityEnforced,
        void *appInstance);

public:
    RComInterthread(void);
    ~RComInterthread(void);
    string getRComStatus(void);
    bool initialize(
        int myID,
        void(*applicationCallbackFunc)(int, const void *, size_t, bool, void *),
        void *extraCallbackFuncArg);
    bool sendData(
        int recipientID,
        const void *data,
        size_t dataSize,
        bool enfoceReliable);
    void setInterRComPort(int port);
    void setPeerRComIPs(const char peerRComIPs[RCOM_NUM_PATHS][IP_ARRAY_SIZE]);
};

inline void RComInterthread::setInterRComPort(int port)
{
    interRComPort_ = port;
}

#endif
/**********************************************************************
 * $Log: RComInterthread.h,v $
 * Revision 1.1  2008-07-15 22:33:59  endo
 * OpenCV, Gazebo and USARSim added.
 *
 * Revision 1.2  2006/08/09 21:46:53  endo
 * -ansi -pedantic added to Makefile
 *
 * Revision 1.1  2006/08/09 21:12:37  endo
 * RCom added.
 *
 * Revision 1.5  2006/04/14 11:24:35  endo
 * RComInterthread::getRComStatus() added.
 *
 * Revision 1.4  2006/02/27 20:41:54  endo
 * rcomserver bug fixed.
 *
 * Revision 1.3  2006/02/25 22:10:23  endo
 * The user can now specify ports to be used.
 *
 * Revision 1.2  2006/02/25 16:17:49  endo
 * rcomserver added
 *
 * Revision 1.1  2006/02/24 11:50:34  endo
 * RComm_Controller checked in.
 *
 **********************************************************************/
