/**********************************************************************
 **                                                                  **
 **                            RComAPI.h                             **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  The interface class for RCom.                                   **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef RComAPI_H
#define RComAPI_H

/* $Id: RComServerInterface.h,v 1.1 2008-07-15 22:34:01 endo Exp $ */

#include <string>

using std::string;

class RComAPI {

public:
    //----------------------------------------------------------------
    // This function will initialize the connection to RCom.
    //----------------------------------------------------------------
    virtual ~RComAPI(void) {}

    //----------------------------------------------------------------
    // This function will initialize the connection to RCom.
    // 
    // Input:
    //  - hostMachine: Either the name of the machine where RCom is running
    //    or the IP address of the machine.
    //  - applicationID: an unique ID that distinguish itself from other
    //    applications. This ID will be used used to deliver the data
    //    to appropriate destiny.
    //
    // Output:
    //  - Returns true if success. Otherwise, false.
    //----------------------------------------------------------------
    virtual bool connectToRCom(
        string hostMachine,
        string port,
        int applicationID) = 0;

    //----------------------------------------------------------------
    // This function will ship the data to RCom.
    // 
    // Input:
    //  - recipientID: ID of the recipient application.
    //  - data: data to be sent.
    //  - dataSize: the size of the data.
    //  - enforceReliable: the flag to notifies RCom that whether this
    //    data should be sent via absolutely reliable connection or
    //    if the loss of the data can be tolerated. 
    //
    // Output:
    //  - Returns true if success. Otherwise, false.
    //----------------------------------------------------------------
    virtual bool sendData(
        int recipientID,
        const void *data,
        size_t dataSize,
        bool enfoceReliable) = 0;

    //----------------------------------------------------------------
    // This function checks to see if data is ready to be read.
    // 
    // Output:
    //  - Returns the size of the data ready to be grabbed.
    //----------------------------------------------------------------
    virtual size_t getReceivedDataSize(void) = 0;

    //----------------------------------------------------------------
    // This function grabs the data being received by RCom.
    // 
    // Input:
    //  - data: copy of the data being received. Memory of the data
    //    has to be allocated before this function is being called.
    //  - dataSize: the size of the data to be grabbed. Use checkDataReady()
    //    function to get the value.
    //  - senderID: this pointer indicating the sender of the data
    //    will be filled by RCom.
    //
    // Output:
    //  - Returns the ID of the sender.
    //----------------------------------------------------------------
    virtual int copyReceivedData(void *data, size_t dataSize) = 0;
};

#endif
/**********************************************************************
 * $Log: RComServerInterface.h,v $
 * Revision 1.1  2008-07-15 22:34:01  endo
 * OpenCV, Gazebo and USARSim added.
 *
 * Revision 1.1  2006/08/09 21:12:37  endo
 * RCom added.
 *
 * Revision 1.1  2006/02/22 07:17:23  endo
 * RComServer and RComClient implemented.
 *
 * Revision 1.2  2006/02/20 16:15:50  endo
 * Typo
 *
 * Revision 1.1  2006/02/20 14:14:48  endo
 * RComAPI checked.
 *
 **********************************************************************/
