/**********************************************************************
 **                                                                  **
 **                         RComAPIFactory.h                         **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  A factor class for RComAPI.                                     **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef RComAPIFactory_H
#define RComAPIFactory_H

/* $Id: RComServerInterfaceFactory.h,v 1.1 2008-07-15 22:34:01 endo Exp $ */

#include <string>
#include <RComAPI.h>
#include <RComClient.h>
#include <RComServer.h>

using std::string;

typedef enum RComAPITypes_t {
    RCOMAPI_TYPE_RCOMCLIENT,
    RCOMAPI_TYPE_RCOMSERVER,
    NUM_RCOMAPI_TYPES
};

//----------------------------------------------------------------
// Sole purpose of this class to instantiate the implementation
// of RComAPI.
//----------------------------------------------------------------
class RComAPIFactory {

public:
    static RComAPI *createRComAPI(int rcomAPIType);
};

inline RComAPI *RComAPIFactory::createRComAPI(int rcomAPIType)
{
    switch (rcomAPIType) {

        case RCOMAPI_TYPE_RCOMCLIENT:
            return (new RComClient());

        case RCOMAPI_TYPE_RCOMSERVER:
            return (new RComServer());
    }

    return NULL;
};

#endif
/**********************************************************************
 * $Log: RComServerInterfaceFactory.h,v $
 * Revision 1.1  2008-07-15 22:34:01  endo
 * OpenCV, Gazebo and USARSim added.
 *
 * Revision 1.1  2006/08/09 21:12:37  endo
 * RCom added.
 *
 * Revision 1.1  2006/02/22 07:17:23  endo
 * RComServer and RComClient implemented.
 *
 * Revision 1.1  2006/02/20 14:14:48  endo
 * RComAPI checked.
 *
 **********************************************************************/
