/**********************************************************************
 **                                                                  **
 **                      RComm_Controller.cpp                        **
 **                                                                  **
 **  Written by:  I.C. Envarli                                       **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef __RComm_Controller_cpp
#define __RComm_Controller_cpp

#include "RComm_Controller.h"

comm_thread_info_t cti[RCOM_NUM_PATHS];

RComm_Controller *gRComm_Controller = NULL;

void * Receiver_Loop_Thread(void * ptr){
	comm_thread_info_t * temp = (comm_thread_info_t * ) ptr;
	temp->r_pointer->Run_Receiver_Loop(temp->pathnumber);
    return NULL;
}

void * Heart_Beat_Thread(void * ptr){
	((Reliable_Comm *)ptr)->Run_Heartbeat_Loop();
    return NULL;
}

void * Thruput_Thread(void *ptr){
  ((Reliable_Comm*)ptr)->UpdateThruputConstantly();
  return NULL;

}
RComm_Controller::RComm_Controller(void) : initialized_(false) {}

bool RComm_Controller::init(char ipnumbers[3][16], int port){
	if (r_comm.Setup_Receiver(port)==false) return false;
	if (r_comm.Setup_Sender(port,ipnumbers)==false) return false;
    initialized_ = true;
	for (int j=0;j<RCOM_NUM_PATHS;j++){
		cti[j].r_pointer= & r_comm;
		cti[j].pathnumber=j;
		pthread_create(&recv_threads[j],	NULL, Receiver_Loop_Thread,(void*)(cti+j));
	}
	pthread_create(&heart_beat_thread,NULL, Heart_Beat_Thread,(void*) & r_comm);
	pthread_create(&thruput_thread,NULL,Thruput_Thread,(void*) & r_comm);
	return true;
}

void RComm_Controller::Send_Packet_Reliable(byte payload[], const unsigned int payload_len, const unsigned int recipient_id){
  r_comm.Send_Packet_Reliable(payload, payload_len, recipient_id);
}

void RComm_Controller::Send_Packet_Unreliable(byte payload[],const unsigned int payload_len, const unsigned int recipient_id){
  r_comm.Send_Packet_Unreliable(payload, payload_len, recipient_id);
}

void RComm_Controller::Print_Stats(){
  r_comm.Print_Stats();
}

void RComm_Controller::Unregister_App(int app_id){
  r_comm.Unregister_App(app_id);
}

bool RComm_Controller::Register_App(Receiver_Callback_p callback_func, int app_id,byte * local_buffer, void *app_instance){
  return r_comm.Register_App(callback_func,app_id,local_buffer, app_instance);
}
const char * RComm_Controller::Get_Stats(){
  return r_comm.Get_Stats();
}
void RComm_Controller::Modify_Inter_Packet_Time(int path,int newspeed_usec){
  r_comm.Modify_Inter_Packet_Time(path,newspeed_usec);
}

#endif //__RComm_Controller_cpp

/**********************************************************************
 * $Log: RComm_Controller.cpp,v $
 * Revision 1.1  2008-07-15 22:33:57  endo
 * OpenCV, Gazebo and USARSim added.
 *
 * Revision 1.1  2006/08/09 21:12:37  endo
 * RCom added.
 *
 * Revision 1.6  2006/04/14 04:39:35  envarli
 * *** empty log message ***
 *
 * Revision 1.5  2006/04/14 02:46:19  envarli
 * *** empty log message ***
 *
 * Revision 1.4  2006/04/08 17:01:30  envarli
 * Added a new thread for thruput calculations, not currently used but will be soon
 *
 * Revision 1.3  2006/04/02 22:46:29  endo
 * Unregister_App() added to Reliable_Comm.
 *
 * Revision 1.2  2006/02/25 22:10:22  endo
 * The user can now specify ports to be used.
 *
 * Revision 1.1  2006/02/24 11:50:33  endo
 * RComm_Controller checked in.
 *
 **********************************************************************/
