/*  

    main.cpp
    Copyright (C) 2009 Georgia Institute of Technology

    Author
    John Rogers
    jgrogers@cc.gatech.edu


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "rovio.h"
#include <GL/glut.h>
#include <cstdlib>
#include <math.h>


#if !defined(GLUT_WHEEL_UP)
#  define GLUT_WHEEL_UP   3
#  define GLUT_WHEEL_DOWN 4
#endif

static float g_lightPos[4] = { 10, 10, -100, 1 };  // Position of light

static int g_Width = 600;                          // Initial window width
static int g_Height = 600;                         // Initial window height
static int g_yClick = 0;
static int g_xClick = 0;

 double g_xGoal = 0;
 double g_yGoal = 0;
 bool g_GoalGiven = false;

static GLfloat g_fViewDistance = 13.0;
static GLfloat g_nearPlane = 0.01;
static GLfloat g_farPlane = 1000;

double look_at_x = 0.0;
double look_at_y = 0.0;
double look_at_z = 0.0;
double global_coordinate_angle = 0.0;
double global_coordinate_axis_x = 0.0;
double global_coordinate_axis_y = 1.0;
double global_coordinate_axis_z = 0.0;
Rovio* rovio;

void RenderMetric(float min_x, float max_x, 
		  float min_y, float max_y, 
		  float min_z, float max_z) {
  glDisable(GL_LIGHTING);
  glColor3f(0,0,0);
  float dist = 0.02;
  float double_dist = 3*dist;
  glBegin(GL_LINES);
  glVertex3f(min_x, 0, 0);
  glVertex3f(max_x,0,0);
  
  for (float i = 0; i < max_x; i+=0.1){ 
    glVertex3f(i, -dist,0);
    glVertex3f(i, dist,0);    
  }
  for (float i = 0; i > min_x; i-=0.1) {
    glVertex3f(i, -dist,0);
    glVertex3f(i, dist,0);
  }
  for (int i = 0;i<max_x;i+=1) {
    glVertex3f(i, -double_dist,0);
    glVertex3f(i, double_dist,0);
  }
  for (int i = 0;i>min_x;i-=1) {
    glVertex3f(i, -double_dist,0);
    glVertex3f(i, double_dist,0);
  }

  glVertex3f(0,min_y,0);
  glVertex3f(0,max_y,0);
  for (float i = 0; i < max_y; i+=0.1){ 
    glVertex3f(-dist, i,0);
    glVertex3f(dist, i,0);
  }
  for (float i = 0; i > min_y; i-=0.1) {
    glVertex3f(-dist, i, 0);
    glVertex3f(dist, i, 0);
  }
  for (int i = 0;i<max_y;i+=1) {
    glVertex3f(-double_dist, i, 0);
    glVertex3f(double_dist, i, 0);
  }
  for (int i = 0;i>min_y;i-=1) {
    glVertex3f(-double_dist, i, 0);
    glVertex3f(double_dist, i, 0);
  }
  glVertex3f(0,0,min_z);
  glVertex3f(0,0,max_z);
  for (float i = 0; i < max_z; i+=0.1){ 
    glVertex3f(-dist,0, i);
    glVertex3f(dist, 0, i);
  }
  for (float i = 0; i > min_z; i-=0.1) {
    glVertex3f(-dist,0, i);
    glVertex3f(dist, 0, i);
  }
  for (int i = 0;i<max_z;i+=1) {
    glVertex3f(-double_dist,0, i);
    glVertex3f(double_dist, 0, i);
  }
  for (int i = 0;i>min_z;i-=1) {
    glVertex3f(-double_dist,0, i);
    glVertex3f(double_dist, 0, i);
  }
  glEnd();
  glEnable(GL_LIGHTING);
}

void InitGraphics(void)
{
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LESS);
   glShadeModel(GL_SMOOTH);
   //   glEnable(GL_LIGHTING);
   //glEnable(GL_LIGHT0);
   //Create texture for cube; load marble texture from file and bind it
}

void Reshape(int width, int height)
{
   g_Width = width;
   g_Height = height;
   glViewport(0, 0, g_Width, g_Height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-g_fViewDistance,g_fViewDistance, -g_fViewDistance, g_fViewDistance, -1000, 1000);
   glMatrixMode(GL_MODELVIEW);
}
void SpecialKeyboard(int key, int x, int y) {
  switch(key) {
  case GLUT_KEY_UP:
    look_at_x += 0.3;
    break;
  case GLUT_KEY_DOWN:
    look_at_x -= 0.3;
    break;
  case GLUT_KEY_PAGE_UP:
    look_at_y += 0.3;
    break;
  case GLUT_KEY_PAGE_DOWN:
    look_at_y -= 0.3;
    break;
  case GLUT_KEY_RIGHT:
    look_at_z += 0.3;
    break;
  case GLUT_KEY_LEFT:
    look_at_z -= 0.3;
    break;
  }
  glutPostRedisplay();
}

void Keyboard(unsigned char key, int x, int y)
{
  switch (key)
  {
  case 27:             // ESCAPE key
  case 'q':
    exit (0);
    break;
  default:
    break;
  }
  glutPostRedisplay();
}

void MouseButton(int button, int state, int x, int y)
{
  if (state == GLUT_UP )
    {
      if ( button == GLUT_WHEEL_UP )
	{
	  g_fViewDistance /=1.5;
	  if (g_fViewDistance < 1.0) g_fViewDistance = 1.0;
	  glutPostRedisplay();
	}
      else if( button == GLUT_WHEEL_DOWN )
	{
	  g_fViewDistance *=1.5;
	  glutPostRedisplay();
	}
    }
  // Respond to mouse button presses.
  // If button1 pressed, mark this state so we know in motion function.
  if (button == GLUT_LEFT_BUTTON)
    {
      //Issue robot control command, or set goal point or something
      g_GoalGiven = true;
      double xw = x / (double)g_Width - 0.5;
      double yh = 0.5 - y / (double)g_Height;
      g_xGoal = 2*g_fViewDistance * yh - look_at_z;
      g_yGoal = 2*g_fViewDistance * xw - look_at_x;
    }
}

void MouseMotion(int x, int y)
{

}
void Display(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-g_fViewDistance,g_fViewDistance, -g_fViewDistance, g_fViewDistance, -1000, 1000);
  glMatrixMode(GL_MODELVIEW);

  // Clear frame buffer and depth buffer
  glClearColor(0.9,0.9,0.9,1.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  // Set up viewing transformation, looking down -Z axis
  glLoadIdentity();
  gluLookAt(look_at_x, 
	    look_at_y, 
	    g_fViewDistance + look_at_z, 
	    look_at_x, look_at_y, look_at_z, 
	    1, 0, 0);
  // Set up the stationary light
  g_lightPos[0]= look_at_x;
  g_lightPos[1]= g_fViewDistance + look_at_y;
  g_lightPos[2]= look_at_z;
  glLightfv(GL_LIGHT0, GL_POSITION, g_lightPos);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  // Render the scene
  //RenderObjects(true);
  RenderMetric(-20,20,-20,20,-20,20);
  double X,Y,Theta;
  rovio->GetPose(X,Y,Theta);
  glTranslatef(X,Y,0.0);
  glRotatef(Theta*180.0/M_PI, 0,0,1);
  glRotatef(90, 1,0,0);
  glutSolidTeapot(0.1);
  glutSwapBuffers();
}
void Idle(void) {
  IplImage* img = rovio->GetImage();
  cvShowImage("image",img);
  cvWaitKey(10);
  glutPostRedisplay();
}

int main(int argc, char** argv) {
  glutInit(&argc, argv);
  if (argc < 2) exit(0);
  rovio = new Rovio(argv[1]);
  cvNamedWindow("image",1);
  glutInitWindowSize (600, 600);
  glutInitDisplayMode ( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutCreateWindow ("Robot Display");
  // Initialize OpenGL graphics state
  InitGraphics();
  // Register callbacks:
  glutDisplayFunc (Display);
  glutReshapeFunc (Reshape);
  glutKeyboardFunc (Keyboard);
  glutSpecialFunc(SpecialKeyboard);
  glutMouseFunc (MouseButton);
  glutMotionFunc (MouseMotion);
  glutIdleFunc(Idle);
  glutMainLoop();
  return 0;
}

