/*  

    rovio.h
    Copyright (C) 2009 Georgia Institute of Technology

    Author
    John Rogers
    jgrogers@cc.gatech.edu


    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#pragma once
#include <pthread.h>
#include <curl/curl.h>
#include <string>

#ifndef NO_OPENCV
#include <highgui.h>
#endif

#include "../hserver.h"




using namespace std;
template<class T, void(T::*mem_fn)()>
  void* thunk(void* p)
{
  (static_cast<T*>(p)->*mem_fn)();
  return 0;
}

class Rovio {
 protected:
  double X;
  double Y;
  double Theta;
  
  double dX;
  double dY;
  double dTheta;

  FrameData CurFrame;
  int NextFrameTime;

  pthread_t mySubThread;
  pthread_mutex_t velocity_mutex;
  pthread_mutex_t curl_mutex;

  char * host;
  CURL* general_curl;
  CURL* mutexed_curl;


  char* errorBuffer;
  CURLcode libcurl_sendcommand(CURL* curl, char* urlstr, 
			       string& buffer);
  void ComputeForwardKinematics(double V_l, double V_r, double V_c,
				double& Vx, double& Vy, double& omega);
  void libcurl_grabframe( char* urlstr);
  void libcurl_getstate(char* urlstr);
  void libcurl_getenc(char* base);
  void libcurl_sendcommand_noresp(char * urlstr);

#ifndef NO_OPENCV
  IplImage* m_pLastImage;
  IplImage* PrepareImage(const IplImage* imgin, 
			 IplImage* imgout);
#endif

  void curl_setup(CURL*& curl);
  void sockerror(const char *msg);

 public:
  Rovio(const char* ip_addr);
  ~Rovio();
  void SetControl(double vx, double vy, double vtheta);
  void GetPose(double& X, double& Y, double& Theta);
  void GetImage(void);
  void PeriodicUpdate();
  FrameData GetCurFrame(void);
  bool getIR();
};
