
#include <string.h>
#include <termio.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <fcntl.h>
#include <math.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "gps.h"
#include "statusbar.h" 
#include "hserver.h"
#include "message.h"
#include "ipt_handler.h"
#include "hserver_ipt.h"
#include "ipt/callbacks.h"
#include "ipt/connection.h"
#include "robot.h"
#include "PoseCalculatorInterface.h"
#include "Watchdog.h"
#include "LogManager.h"
#include "cricketd/NetComm.h"
#include "cricketd/trilateration.h"

extern "C" {
#include "cricketd/cricketd.h"
}

#include "cricket.h"

declareHandlerCallback(Cricket)
implementHandlerCallback(Cricket)

pthread_mutex_t cricketPositionMutex;
double cricketEnabled = false;
HSPose_t cricketPosition;
bool cricketGotValidYaw = false;



// get sockaddr, IPv4 or IPv6:
void *get_in_addr(struct sockaddr *sa) {
  if (sa->sa_family == AF_INET) {
    return &(((struct sockaddr_in*)sa)->sin_addr);
  }
  return &(((struct sockaddr_in6*)sa)->sin6_addr);
}


const int Cricket::WATCHDOG_CHECK_INTERVAL_SEC_ = 10;

void Cricket::updateStatusBar_(void)
{

    //double curtime = getCurrentEpochTime();

    sprintf(statusStr, "Cricket: X = %f, Y = %f, Z = %f", xyz[0], xyz[1], xyz[2]);

    statusbar->update(statusLine);

}


void Cricket::addSensors()
{
    partCricket = new RobotPartCricket();

    sensorCricket = new Sensor(
        SENSOR_CRICKET,
        0,
        NULL,
        0,
        NULL,
        0,    // Changed from 3,xyz
        NULL, 
        &mutexStatsLock);
}



static void start_thread(void* cricket_instance)
{
    ((Cricket*) cricket_instance)->reader_thread();
}

static void start_cricket_thread(void* cricket_instance)
{
    ((Cricket*) cricket_instance)->cricket_thread();
}

void Cricket::cricket_thread() {
  char result[10000];
  bool gotvalue = false;
  int dbval;
  FILE *in = NULL;
  beaconrecord br;
  beaconposition bp;
  char *tresult = NULL;
  double timestamp;
  HSPose_t pose;
  bool gotValidYaw = false;
  double calculatedYaw = 0.0;

  static FILE *cdebug = fopen("cricket.log", "a");

  // PARAMETERS (fitting function)
  double sonarscale =  1;//1.076902;  //1  if previously corrected in PositionServer*)
  double sonaroffset = 0;//-4.4617;  // 0 if prev. corrected in PositionServer*)

  Trilateration *tri = new Trilateration();

  NetComm *comm = new NetComm();
  comm->setHost("192.168.1.80");
  comm->setPort(3001);

  comm->init();
  comm->write_("register\n\r");

  while (1) {
    cricketEnabled = true;

    comm->read_(result);
    gotvalue = true;

    fprintf(cdebug, "%lf: %s\n", getCurrentEpochTime, result);
    fflush(cdebug);

    if ( (tresult = strstr(result, "DB=")) != NULL) {
      sscanf(tresult, "DB=%d", &dbval);

      br.db = (double) dbval;

      // Apply fitting function
      br.db = (br.db * sonarscale) + sonaroffset;

    } else {
      gotvalue = false;
    }

    if ( (tresult = strstr(result, "SP=")) != NULL) {
      sscanf(tresult, "SP=BEACON-%d", &br.beaconid);
    } else {
      gotvalue = false;
    }

    if ( (tresult = strstr(result, "TS=")) != NULL) {
       sscanf(tresult, "TS=%lf", &br.timestamp);
       timestamp = br.timestamp;
    } else {
       gotvalue = false;
    }

    if (gotvalue) {
       if (tri->addData(br) == false) {
          fprintf(stderr, "%.0lf: Could not add beacon reading\n", timestamp);
       } else {
          //printf("%.0lf: Added %d,%lf\n", timestamp, br.beaconid, br.db);
       }

       if (tri->getPosition(bp) == true) {
          //printfTextWindow("%.0lf: Received Position (%lf,%lf,%lf)\n", timestamp, bp.x, bp.y, bp.z);
          xyz[0] = bp.x/100.0; /*+ poseOffsetX;*/
          xyz[1] = bp.y/100.0; /*+ poseOffsetY;*/
          xyz[2] = bp.z/100.0;

          // send position to posecalc
          if ((gPoseCalc != NULL) && (poseCalcID_ >= 0)) {
             pose.loc.x = xyz[0];
             pose.loc.y = xyz[1];
             pose.loc.extra.time = getCurrentEpochTime();
             //gPoseCalc->updateModuleLocation(poseCalcID_, pose.loc);

             calculatedYaw = calculate_yaw(pose, gotValidYaw);
             if (gotValidYaw) {
               pose.rot.extra.time = getCurrentEpochTime();
               pose.rot.yaw = calculatedYaw;
               //gPoseCalc->updateModuleRotation(poseCalcID_, pose.rot);
             }

             pthread_mutex_lock(&cricketPositionMutex);
             cricketPosition = pose;
             cricketGotValidYaw = gotValidYaw;
             printfTextWindow("Cricket valid yaw %d\n", gotValidYaw);
             pthread_mutex_unlock(&cricketPositionMutex);

             updateStatusBar_();
          }
       } else {
          //printfTextWindow("%.0lf: No position\n", timestamp);
       }
    }
    usleep(1000);
  }
}

// This function calculates an estimated orientation (yaw) from consecutive
// cricket readings.  It does this by maintaining a running average over a
// fixed window size (to filter out noisy data) and then estimates the
// general direction of motion.
double Cricket::calculate_yaw(HSPose_t &pose, bool &gotValidYaw) {

   // Parameters: Window size & distance threshold
   static unsigned int historyWindow = 10; //10;
   static double dist_threshold = 0.15 * 0.15; // Squared for quicker euclidean distance calculation

   static vector <double> lastX;
   static vector <double> lastY;
   double avgX1=0.0, avgX2=0.0;
   double avgY1=0.0, avgY2=0.0;
   double ryaw=0.0, ayaw=0.0;
   double currX=0.0, currY=0.0;
   double window1start = 0, window1end = historyWindow;
   double window2start = historyWindow, window2end = lastX.size();

   currX = pose.loc.x;
   currY = pose.loc.y;

   if (lastX.size() < (historyWindow*2) ) {
     gotValidYaw = false;

     lastX.push_back(currX);
     lastY.push_back(currY);

     return ayaw;
   }

   // Calculate average over previous window
   for (unsigned int i = window1start; i < window1end; i++) {
     avgX1 += lastX[i];
     avgY1 += lastY[i];
   }
   avgX1 = avgX1 / lastX.size();
   avgY1 = avgY1 / lastY.size();

   // Calculate average over current window (note i starts from 1 not 0)
   for (unsigned int i = window2start; i < window2end; i++) {
     avgX2 += lastX[i];
     avgY2 += lastY[i];
   }

   // Add latest reading to new running average
   //avgX2 += currX; 
   //avgY2 += currY;
   avgX2 = avgX2 / lastX.size();
   avgY2 = avgY2 / lastY.size();

   // We haven't really moved much, yaw is probably noise due to noisy cricket readings 
   if ( ((avgX1 - avgX2) * (avgX1 - avgX2) + (avgY1 - avgY2) * (avgY1 - avgY2)) < dist_threshold ) {
     gotValidYaw =  false;
   } else {

     // Calculate yaw from difference between two running averages 
     ryaw = atan2( (avgY2 - avgY1), (avgX2 - avgX1) );

     // Convert to degrees
     ayaw = ROBOT_RAD2DEG(ryaw);

     gotValidYaw = true;
     printfTextWindow("(%lf,%lf) => (%lf,%lf)  yaw=%lf\n", avgX1, avgY1, avgX2, avgY2, ayaw);
   }

   lastX.erase(lastX.begin());
   lastY.erase(lastY.begin());

   lastX.push_back(currX);
   lastY.push_back(currY);
  
   return ayaw;
}

void Cricket::reader_thread()
{
    struct timeb tp;
    static bool curupdate = false;
    char inbuf[100];
    HSPose_t pose;
    double timethreshold = 500;

    bool gotValidYaw = false;
    bool gotValidPose = false;
    double poseOffsetX = 0.0;  // ZK CRICKET TRANSFORMATION 
    double poseOffsetY = 0.0;

    char idbuf[5];
    idbuf[0] = idbuf[1] = idbuf[2] = idbuf[3] = idbuf[4] = 0;
    sprintf(idbuf, "%d", RobotId);
    int idlen = strlen(idbuf);

    while (true) {

      // update the cricket at 2Hz
      ftime(&tp);
      if ( (tp.millitm < timethreshold) && !curupdate) {
        curupdate = true;
        double x=0.0,y=0.0,z=0.0;

        // Lock semaphore

        // Obtain position

        // Unlock semaphore

        if (gotValidPose) {
            xyz[0] = x + poseOffsetX;
            xyz[1] = y + poseOffsetY;
            xyz[2] = z;


            // send position to posecalc
            if ((gPoseCalc != NULL) && (poseCalcID_ >= 0)) {
             pose.loc.x = xyz[0];
             pose.loc.y = xyz[1];
             pose.loc.extra.time = getCurrentEpochTime();
             //gPoseCalc->updateModuleLocation(poseCalcID_, pose.loc);

             pose.rot.yaw = calculate_yaw(pose, gotValidYaw);
             if (0&&gotValidYaw) {
               pose.rot.extra.time = getCurrentEpochTime();
               gPoseCalc->updateModuleRotation(poseCalcID_, pose.rot);
             }
             updateStatusBar_();

         }
         else {
           printfTextWindow("No position available");
         }

        }
      }
   }
}



Cricket::~Cricket()
{
    if ((gPoseCalc != NULL) && (poseCalcID_ >= 0))
    {
        gPoseCalc->disconnectModule(poseCalcID_);
        poseCalcID_ = -1;
    }

    pthread_cancel(reader);
    pthread_join(reader, NULL);
    pthread_cancel(cricket_reader);
    pthread_join(cricket_reader, NULL);
    free(Port);
    free(IP);
}

Cricket::Cricket(
    Cricket** a,
    const string& strPortString, int id) :
    Module((Module**) a, HS_MODULE_NAME_CRICKET),
    poseCalcID_(-1)
{
    RobotId = id;
    pthread_mutex_init(&mutexStatsLock, NULL);
    pthread_mutex_init(&cricketPositionMutex, NULL);

    xyz[0] = 0.0f;
    xyz[1] = 0.0f;
    xyz[2] = 0.0f;
    addSensors();
    updateStatusBar_();

    const char *ipadd = strPortString.c_str();

    int len = strlen(ipadd);    
    char *portstr = strstr(ipadd, ":");
    if (portstr == NULL) {
        printfTextWindow("IP parameter should be formatted as: x.x.x.x:p");
        exit(1);
    }
    int portlen = strlen(portstr);
    char buf[20];
    strcpy(buf, &portstr[1]);
    Port = strdup(buf);
    strncpy(buf, ipadd, len - portlen);
    buf[len - portlen] = 0;
    IP = strdup(buf);

    //pthread_create(&reader, NULL, (void*(*)(void*))&start_thread, (void*) this);
    pthread_create(&cricket_reader, NULL, (void*(*)(void*))&start_cricket_thread, (void*) this);

}



