/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
#include "config.h"
#include <stdlib.h>
#include <string.h>

#if defined (HAVE_STRINGS_H)
#include <strings.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
//#include <varargs.h>
#include <netdb.h>
#include <stdio.h>
#include <arpa/inet.h>

#if defined (HAVE_SYS_PARAM_H)
#include <sys/param.h>
#endif

#include "cricketd.h"


#if !defined (INADDR_NONE)
#define INADDR_NONE   ((in_addr_t)-1)
#endif

static char mbuf[128];

int passivesock(char *service, char *protocol, int qlen)
{
    struct servent *pse;
    struct protoent *ppe;
    struct sockaddr_in sin;
    int s, type;
    int one = 1;

    bzero((char *) &sin, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;

    if ( (pse = getservbyname(service, protocol)) )
	sin.sin_port = htons(ntohs((u_short) pse->s_port));
    else if ((sin.sin_port = htons((u_short) atoi(service))) == 0) {
	sprintf(mbuf, "Can't get \"%s\" service entry.\n", service);
	errexit(mbuf);
    }
    if ((ppe = getprotobyname(protocol)) == 0) {
	sprintf(mbuf, "Can't get \"%s\" protocol entry.\n", protocol);
	errexit(mbuf);
    }
    if (strcmp(protocol, "udp") == 0)
	type = SOCK_DGRAM;
    else
	type = SOCK_STREAM;

    s = socket(PF_INET, type, ppe->p_proto);
    if (s < 0)
	errexit("Can't create socket:");

    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) == -1) {
        sprintf(mbuf, "%s", "Error: SETSOCKOPT SO_REUSEADDR");
	errexit(mbuf);
    }
    if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
	sprintf(mbuf, "Can't bind to port %s", service);
	errexit(mbuf);
    }
    if (type == SOCK_STREAM && listen(s, qlen) < 0) {
	sprintf(mbuf, "Can't listen on %s port:", service);
	errexit(mbuf);
    }

    //printf("listening for conns on: %s\n", service);
    return s;
}

int passiveTCP(char *service, int qlen)
{
    return passivesock(service, "tcp", qlen);
}


int connectsock(char *host, char *service, char *protocol)
{
 int sockfd, new_fd;  // listen on sock_fd, new connection on new_fd
 struct addrinfo hints, *servinfo, *p;
    struct sockaddr_storage their_addr; // connector's address information
    socklen_t sin_size;
    int yes=1;
    char s[INET6_ADDRSTRLEN];
    int rv;

    int ListenSocket;
    
    memset(&hints, 0, sizeof hints);
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE; // use my IP

    // char portbuf[30];
    //sprintf(portbuf, "%d", ListenPort);

    if ((rv = getaddrinfo(host, service, &hints, &servinfo)) != 0) {
        fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(rv));
        return;
    }

    // loop through all the results and bind to the first we can
    for(p = servinfo; p != NULL; p = p->ai_next) {
        if ((ListenSocket = socket(p->ai_family, p->ai_socktype,
                p->ai_protocol)) == -1) {
	    printf("Could not create PositionServer listen socket\n");
            exit(1);
        }
	//	fcntl(ListenSocket, F_SETFL, O_NONBLOCK);

        if (setsockopt(ListenSocket, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1) {
	    printf("Could not set PositionServer listen socket options\n");
            exit(1);
        }

        if (bind(ListenSocket, p->ai_addr, p->ai_addrlen) == -1) {
            close(ListenSocket);
            printf("Could not bind to PositionCalculator Listen socket, Port: %s\n", service);
	    exit(1);
        }

        break;
    }

    if (p == NULL)  {
        fprintf(stderr, "server: failed to bind\n");
        exit(1);
    }

    freeaddrinfo(servinfo); // all done with this structure

    if (listen(ListenSocket, 1) == -1) {
        perror("listen");
        exit(1);
    }


    /*
    struct hostent *phe;
    struct servent *pse;
    struct protoent *ppe;
    struct sockaddr_in sin, tin;
    int s, type, con;
    int one = 1;

    bzero((char *) &sin, sizeof(sin));
    sin.sin_family = AF_INET;

    if ( (pse = getservbyname(service, protocol)) )
	sin.sin_port = htons(ntohs((u_short) pse->s_port));
    else if ((sin.sin_port = htons((u_short) atoi(service))) == 0) {
	sprintf(mbuf, "Can't get \"%s\" service entry.\n", service);
	errexit(mbuf);
    }
    if ( (phe = gethostbyname(host)) )
	bcopy(phe->h_addr, (char *) &sin.sin_addr, phe->h_length);
    else if ((sin.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE) {
	sprintf(mbuf, "Can't get host entry: \"%s\".\n", host);
	errexit(mbuf);
    }
    if ((ppe = getprotobyname(protocol)) == 0) {
	sprintf(mbuf, "Can't get \"%s\" protocol entry.\n", protocol);
	errexit(mbuf);
    }
    if (strcmp(protocol, "udp") == 0)
	type = SOCK_DGRAM;
    else
	type = SOCK_STREAM;

    s = socket(PF_INET, type, ppe->p_proto);
    if (s < 0)
	errexit("Can't create socket:");

    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) == -1) {
        sprintf(mbuf, "%s", "Error: SETSOCKOPT SO_REUSEADDR");
	errexit(mbuf);
    }
    if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
        sprintf(mbuf, "Can't bind to port %s", service);
        errexit(mbuf);
    }
    if (listen(s, 1) == -1) {
        printf("Can not listen for connectport");
    }

    fprintf(stderr, "Waiting for connection from connectport\n");
    socklen_t tin_size;
    tin_size = sizeof tin;
    con = accept(s, (struct soccaddr *) &tin, &tin);
    fprintf(stderr, "Got connection from connectport on port: %d\n", con);
    if (con == -1) {
       perror("accept");
    }

//    if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
//	sprintf(mbuf, "Can't connect to %s.%s", host, service);
//	errexit(mbuf);
//    }
*/


      struct sockaddr_storage remoteaddr;       
//	struct sockaddr remoteaddr;
      socklen_t addrlen = sizeof remoteaddr;
      int con = accept(ListenSocket, (struct sockaddr *)&remoteaddr, &addrlen);

      //fprintf(stderr, "Waiting for connection from connectport\n");
      //fprintf(stderr, "Got connection from connectport on port: %d\n", con);
      if (con == -1) {
	perror("accept");
      }

    return con;


}

int connectTCP(char *host, char *service)
{
    return connectsock(host, service, "tcp");
}
