#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <vector>

#include "trilateration.h"
#include "fromToRotation.h"
#include "circle_circle.h"

using namespace std;


Trilateration::Trilateration() {
  timestamp = 0.0;

  debug = 0;
  errordebug = false;
}


// Perform transformation (represented as a matrix) on vector 
beaconposition Trilateration::transformVector(beaconposition bp, mxn_Matrix *matrix) {
	beaconposition result;
	result.x = matrix->Get_RowCol(0,0) * bp.x + matrix->Get_RowCol(0,1) * bp.y + matrix->Get_RowCol(0,2) * bp.z + matrix->Get_RowCol(0,3);
	result.y = matrix->Get_RowCol(1,0) * bp.x + matrix->Get_RowCol(1,1) * bp.y + matrix->Get_RowCol(1,2) * bp.z + matrix->Get_RowCol(1,3);
	result.z = matrix->Get_RowCol(2,0) * bp.x + matrix->Get_RowCol(2,1) * bp.y + matrix->Get_RowCol(2,2) * bp.z + matrix->Get_RowCol(2,3);
	return result;
}

// Code to calculated 3D position based on 2 or 3 known beacon locations
// Based on code by Tom Collins in Mathematica
bool Trilateration::addData(beaconrecord br) {
  beaconhistory.push_back(br);
  timestamp = br.timestamp;
  return true;
}

bool Trilateration::getPosition(beaconposition &bp) {  

   static mxn_Matrix *tt = new mxn_Matrix(4,4);
   static mxn_Matrix *tr1 = new mxn_Matrix(4,4);
   static mxn_Matrix *tr2 = new mxn_Matrix(4,4);
   static mxn_Matrix TT(4,4);
   static mxn_Matrix IT(4,4);

   static mxn_Matrix *M1 = new mxn_Matrix(3,3);
   static mxn_Matrix *M2 = new mxn_Matrix(3,3);

  /************************ PARAMETERS ********************************/

  //At times, I eliminated some solutions as being completely inconsistent with a possible 
  //height range of the receiver.  For a Rovio, that might be -0  to 50 cm. But I haven't used it lately ,
  //instead keeping it wide enough to not exclude many solutions. *)
  int minZ = -20; 

  int maxZ = 500;

  double minJ = 150.0;

  // This is important.  It eliminates some past beacon readings as being too old 
  //It would make the most sense if this were a function of either the robot's current velocity or at least the maximum 
  //possible velocity. *)
  int mintimestep = 1000;	

  /*************************************************************************/
  
  beaconrecord br, br1, br2, br3;
  beaconposition b1, b2, b3;
  beaconposition calculatedPosition;
  double r1, r2, r3;
  double d,i,j;
  double sqrtval;

  //IMPORTANT:  this is where we see if this completes a useful set of beacons for Trilateration 
  //So start with an empty set of beacons and try to find 2-3 usable recent readings *)
  vector <beaconrecord> usefulBeacons;

  if (debug > 0) {
	printf("%lf: %d beacon readings in history\n", timestamp, beaconhistory.size());
  }

  int beaconCount = 0;

  // Clear list
  while (beaconhistory.size() > (mintimestep)) {
    beaconhistory.pop_front();
  }
 
  //TODO: Order of this seems to matter (even if same beacons are picked anyways)
  // Would averaging all possible permutations help?
  //for (int i = (beaconhistory.size()-1); i >= 0; i--) {
  //	br = beaconhistory[i];
  list<beaconrecord>::reverse_iterator brit;
  for (brit = beaconhistory.rbegin(); brit != beaconhistory.rend() ; brit++) {
	br = *brit;

	beaconCount++;

	vector<beaconrecord>::iterator brit2;
	bool found = false;
	for (brit2 = usefulBeacons.begin(); brit2 != usefulBeacons.end(); brit2++) {
	  if ( (*brit2).beaconid == br.beaconid ) {
			found = true;
			break;
		}
	}

	if (found) {
		continue;
	}

	if ( (timestamp-br.timestamp) < mintimestep ) {
		if (debug > 0) {
			printf("%d: ", beaconCount);
			br.print();
		}

		usefulBeacons.push_back(br);
	}

	// TODO: Maybe add all timestamp-valid beacons and use statistics/other info
	// to choose best ones?
	if (usefulBeacons.size() >= 3) {
		break;
	}
  }

  if (usefulBeacons.size() == 3) {

	if (debug == 3) {
		printf("\n=====================================\n");
		printf("%lf: Beacon readings (%lf,%lf,%lf)\n", timestamp, usefulBeacons[0].db, usefulBeacons[1].db, usefulBeacons[2].db);
	}

	if (debug > 0) {
		printf("%lf: %d useful beacons (%d,%d,%d)\n", timestamp, usefulBeacons.size(), usefulBeacons[0].beaconid, usefulBeacons[1].beaconid, usefulBeacons[2].beaconid);
	}

	b1 = beaconposition(beaconloc[usefulBeacons[0].beaconid][0], beaconloc[usefulBeacons[0].beaconid][1], beaconloc[usefulBeacons[0].beaconid][2]);
	b2 = beaconposition(beaconloc[usefulBeacons[1].beaconid][0], beaconloc[usefulBeacons[1].beaconid][1], beaconloc[usefulBeacons[1].beaconid][2]);
	b3 = beaconposition(beaconloc[usefulBeacons[2].beaconid][0], beaconloc[usefulBeacons[2].beaconid][1], beaconloc[usefulBeacons[2].beaconid][2]);

	br1 = usefulBeacons[0]; 
	br2 = usefulBeacons[1]; 
	br3 = usefulBeacons[2]; 

	r1 = (double) br1.db;
	r2 = (double) br2.db;
	r3 = (double) br3.db;

	// Create translation position
	tt->translationMatrix(-beaconloc[br1.beaconid][0], -beaconloc[br1.beaconid][1], -beaconloc[br1.beaconid][2]);
	
	// Translate to this new system (beacon 1 is origin)
	b1 = transformVector(b1, tt);
	b2 = transformVector(b2, tt);
	b3 = transformVector(b3, tt);

	if (debug == 3) {
		printf("TRANSLATION: ");
		tt->Print_Matrix();

		printf("RESULT: ");
		b1.print();
		b2.print();
		b3.print();
	}

	// rotate x axis to point toward second beacon location

	float from[3], to[3], matrix[3][3];
	beaconposition nb;
	nb = b2;
	nb.normalize();
	to[0] = nb.x; to[1] = nb.y; to[2] = nb.z;
	from[0] = 1.0; from[1] = 0.0; from[2] = 0.0;
	fromToRotation(to, from, matrix);

	if (debug == 3) printf("From %f,%f,%f   To %f,%f,%f\n", to[0], to[1], to[2], from[0], from[1], from[2]);

	tr1->clear();
	tr1->Assign_RowCol(3,3, 1.0);
	tr1->Assign_RowCol(0,0, matrix[0][0]);
	tr1->Assign_RowCol(0,1, matrix[0][1]);
	tr1->Assign_RowCol(0,2, matrix[0][2]);

	tr1->Assign_RowCol(1,0, matrix[1][0]);
	tr1->Assign_RowCol(1,1, matrix[1][1]);
	tr1->Assign_RowCol(1,2, matrix[1][2]);

	tr1->Assign_RowCol(2,0, matrix[2][0]);
	tr1->Assign_RowCol(2,1, matrix[2][1]);
	tr1->Assign_RowCol(2,2, matrix[2][2]);

	// Apply rotation matrix tr1
	b1 = transformVector(b1, tr1);
	b2 = transformVector(b2, tr1);
	b3 = transformVector(b3, tr1);

	if (debug == 3) {
		printf("\nROTATION1: ");
		tr1->Print_Matrix();
		printf("\n****************************************************\n");

		printf("RESULT: ");
		b1.print();
		b2.print();
		b3.print();
	}

	// rotate around x axis to bring third beacon into z = 0 plane 
	tr2->rotationMatrixAboutX(-atan2(b3.y, b3.z));
	
	// Compose all transformation matrices
	TT = (*tr2);
	TT = TT * (*tr1);
	TT = TT * (*tt);

	// Reset beacon positions
	b1 = beaconposition(beaconloc[usefulBeacons[0].beaconid][0], beaconloc[usefulBeacons[0].beaconid][1], beaconloc[usefulBeacons[0].beaconid][2]);
	b2 = beaconposition(beaconloc[usefulBeacons[1].beaconid][0], beaconloc[usefulBeacons[1].beaconid][1], beaconloc[usefulBeacons[1].beaconid][2]);
	b3 = beaconposition(beaconloc[usefulBeacons[2].beaconid][0], beaconloc[usefulBeacons[2].beaconid][1], beaconloc[usefulBeacons[2].beaconid][2]);

	// Apply transformation matrix TT
	b1 = transformVector(b1, &TT);
	b2 = transformVector(b2, &TT);
	b3 = transformVector(b3, &TT);

	if (debug == 3) {
		printf("ROTATION2: ");
		tr2->Print_Matrix();
		printf("FINAL: ");

		TT.Print_Matrix();
		printf("RESULT: ");
		b1.print();
		b2.print();
		b3.print();
	}

	// same defs of d, i, j used in Wikipeida diagram 
	d = b2.x;
	i = b3.x;
	j = b3.y;

	if (debug == 3) printf("d=%lf, i=%lf, j=%lf  r1=%lf r2=%lf r3=%lf\n", d, i, j, r1, r2, r3);

	//The third beacon needs to be at least 1.5 m off the axis of the other two
	//This could be a parameter varied by some rationale.  But in our lab, all that matters is that this
	//is small enough to eliminate the essentially colinear combinations of beacons, which explain many
	//of the outlier points in the MIT solution.
	if (fabs(j) > minJ ) {
		calculatedPosition.x = ((r1*r1)-(r2*r2)+(d*d))/(2*d);
		calculatedPosition.y = (r1*r1-r3*r3+i*i+j*j)/(2*j)-i*calculatedPosition.x/j;
		sqrtval = r1*r1-calculatedPosition.x*calculatedPosition.x-calculatedPosition.y*calculatedPosition.y;
		if (sqrtval > 0.0) {
			calculatedPosition.z = sqrt(sqrtval);
			
			if (debug == 3) printf("%lf: xyz: %lf,%lf,%lf\n", timestamp, calculatedPosition.x, calculatedPosition.y, calculatedPosition.z);

			// Store r for 2-beacon solution to use
			IT = TT.Inverse_GJ();

			if (debug == 3) {
				printf("\nInverted matrix: \n");
				IT.Print_Matrix();
			}

			lastr = rval;
			rval = transformVector(calculatedPosition, &IT);

			if (rval.z > minZ && rval.z < maxZ) {	
				calculatedPosition.x = rval.x;
				calculatedPosition.y = rval.y;
				calculatedPosition.z = rval.z;

				// Modify input argument for caller of this function
				bp.x = calculatedPosition.x;
				bp.y = calculatedPosition.y;
				bp.z = calculatedPosition.z;
				
				if (debug > 0) printf("%lf: 3-beacon Position: %lf,%lf,%lf\n", timestamp, calculatedPosition.x, calculatedPosition.y, calculatedPosition.z);
				
				return true;
			} else { 
				if (errordebug) fprintf(stderr, "Z is not within bounds (z=%lf)\n", rval.z);
			}

		} else {
			if (errordebug) fprintf(stderr, "Imaginary z\n");
		}
	} else {
		if (errordebug) fprintf(stderr, "Too colinear\n");
	}

	return false;
  } else if (usefulBeacons.size() == 2) {

	//This is simpler than Trilateration.  Skip the coordinate transformations and get the range data 

	if (debug >= 2) {
		printf("%lf: Beacon readings (%lf,%lf)\n", timestamp, usefulBeacons[0].db, usefulBeacons[1].db);
		printf("lastr={%lf,%lf,%lf)\n", lastr.x, lastr.y, lastr.z);
	 }

	if (debug > 0) {
		printf("%lf: %d useful beacons: %d,%d)\n", timestamp, usefulBeacons.size(), usefulBeacons[0].beaconid, usefulBeacons[1].beaconid);
	}

	br1 = usefulBeacons[0]; 
	br2 = usefulBeacons[1]; 

	r1 = br1.db;
	r2 = br2.db;

	b1 = beaconposition(beaconloc[usefulBeacons[0].beaconid][0], beaconloc[usefulBeacons[0].beaconid][1], beaconloc[usefulBeacons[0].beaconid][2]);
	b2 = beaconposition(beaconloc[usefulBeacons[1].beaconid][0], beaconloc[usefulBeacons[1].beaconid][1], beaconloc[usefulBeacons[1].beaconid][2]);

	// The two solutions lie on intersection of two circles (formed by projecting the spheres on to the
	// known z plane)
	double c1a, c1b, c1c, c2a, c2b, c2c, c1x, c1y, c1r, c2x, c2y, c2r;
	double sol1x, sol1y, sol2x, sol2y;

	// First, obtain constants for equation x^2 + y^2 + ax + by + c = 0
    c1a = -2*b1.x;
	c1b = -2*b1.y;
	c1c =  - (r1*r1) + b1.x*b1.x + b1.y*b1.y + (lastr.z - b1.z)*(lastr.z - b1.z);

    c2a = -2*b2.x;
	c2b = -2*b2.y;
	c2c =  - (r2*r2) + b2.x*b2.x + b2.y*b2.y + (lastr.z - b2.z)*(lastr.z - b2.z);

	if (debug >= 2)	printf("Constants: a=%lf b=%lf c=%lf   circle2: a=%lf b=%lf c=%lf\n", c1a, c1b, c1c, c2a, c2b, c2c);

	// Second, calculate the corresponding radius & center of the two circles
	c1x = -c1a/2.0;
    c1y = -c1b/2.0;
    c1r = c1x*c1x + c1y*c1y - c1c;

	if (c1r < 0) {
		if (errordebug) (stderr, "%lf: Circle 1 has a negative radius (%lf)!\n", timestamp, c1r);
		return false;
	}

	c2x = -c2a/2.0;
    c2y = -c2b/2.0;
    c2r = c2x*c2x + c2y*c2y - c2c;

	if (c2r < 0) {
		if (errordebug) fprintf(stderr, "%lf: Circle 2 has a negative radius (%lf)!\n", timestamp, c2r);
		return false;
	}

	c1r = sqrt(c1r);
	c2r = sqrt(c2r);

	if (debug >= 2) printf("Circles: x1=%lf y1=%lf r1=%lf   circle2: x2=%lf y2=%lf r2=%lf\n", c1x, c1y, c1r, c2x, c2y, c2r);

	// Finally, calculate intersection points
	if (circle_circle_intersection(c1x, c1y, c1r,
								   c2x, c2y, c2r,
								   &sol1x, &sol1y, &sol2x, &sol2y) == 0) {
		if (errordebug) fprintf(stderr, "%lf: No solution to two circle intersection\n", timestamp);		
		return false;
	} else {
		calculatedPosition.clear();

		// Out of the two solutions, choose the one that's closes to previous x,y
		if ( ((sol1x - lastr.x)*(sol1x - lastr.x) + (sol1y - lastr.y)*(sol1y - lastr.y)) < 
			 ((sol2x - lastr.x)*(sol2x - lastr.x) + (sol2y - lastr.y)*(sol2y - lastr.y)) ) {
			calculatedPosition.x = sol1x;
			calculatedPosition.y = sol1y;
			calculatedPosition.z = lastr.z;
		} else {
			calculatedPosition.x = sol2x;
			calculatedPosition.y = sol2y;
			calculatedPosition.z = lastr.z;
		}
		
		if (debug > 0) printf("%lf: 2-beacon Position: %lf,%lf,%lf\n", timestamp, calculatedPosition.x, calculatedPosition.y, calculatedPosition.z);

		bp.x = calculatedPosition.x;
		bp.y = calculatedPosition.y;
		bp.z = calculatedPosition.z;

	}
	return true;
  } else {
	return false;
  }

  return false;
}
