/**********************************************************************
 **                                                                  **
 **                              HSRCFile.h                          **
 **                                                                  **
 **                                                                  **
 **  Written by:  Brian Lee                                          **
 **                                                                  **
 **  Copyright 2002 - 2006, Georgia Tech Research Corporation        **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: HSRCFile.h,v 1.1.1.1 2008-07-14 16:44:19 endo Exp $ */

#ifndef HSRCFILE_H
#define HSRCFILE_H

#include <string>
#include "HServerConfig.h"
#include "RCFileReader.h"
#include "LabelToIdConversions.h"

class ClHSRCFile
{
public:
    ClHSRCFile();
    ClHSRCFile( string strFileName );

    void SetFileName( string strFileName );

    bool CreateConfiguration( ClHServerConfig& clConfig );
    bool CreateConfiguration( const string& strFileName, ClHServerConfig& clConfig );

protected:
    string m_strFileName;
    int m_iCurrentSection;

    int LabelToId( const SuLabelIdPair* asuLabelIdPairs, const string& strLabel );
    bool ProcessLine( const SuRCFileLine& suLine, ClHServerConfig& clConfig, int iLineNum );
    bool ProcessStartSectionLine( const SuRCFileLine& suLine, ClHServerConfig& clConfig, int iLineNum );
    bool ProcessValueLine( const SuRCFileLine& suLine, ClHServerConfig& clConfig, int iLineNum );
};

inline void ClHSRCFile::SetFileName( string strFileName )
{
    m_strFileName = strFileName;
}

inline bool ClHSRCFile::CreateConfiguration( const string& strFileName, ClHServerConfig& clConfig )
{
    SetFileName( strFileName );
    return CreateConfiguration( clConfig );
}

#endif // ifndef HSRCFILE_H

/**********************************************************************
# $Log: HSRCFile.h,v $
# Revision 1.1.1.1  2008-07-14 16:44:19  endo
# MAST Project (based on MissionLab-MINOS-20071018.tar.gz)
#
# Revision 1.1.1.1  2006/07/20 17:17:48  endo
# MINOS Project (based on MissionLab-7.0.20060712.tar.gz)
#
# Revision 1.1.1.1  2006/07/12 13:37:57  endo
# MissionLab 7.0
#
# Revision 1.2  2006/06/05 21:35:54  endo
# PoseCalculator from MARA 2020 migrated
#
#**********************************************************************/
