/**********************************************************************
 **                                                                  **
 **                         rcom_handler.h                           **
 **                                                                  **
 **                                                                  **
 **  Written by: Yoichiro Endo                                       **
 **                                                                  **
 **  Copyright 2006 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: rcom_handler.h,v 1.1 2008-07-15 22:33:52 endo Exp $ */

#ifndef RCOM_HANDLER_H
#define RCOM_HANDLER_H

#include <pthread.h>
#include <time.h>
#include <string>

#include "module.h"
#include "RComAPI.h"

using std::string;

class RComHandler : public Module {

    // Make sure these match with the ones in 
    // air.behavioral.action.ActionType.java.
    typedef enum ActionType_t
    {
        ACTION_TYPE_TRANS_SPPED_X = 0,
        ACTION_TYPE_TRANS_SPPED_Y = 1,
        ACTION_TYPE_TRANS_SPPED_Z = 2,
        ACTION_TYPE_ANG_SPPED_YAW = 3,
        ACTION_TYPE_ANG_SPPED_PITCH = 4,
        ACTION_TYPE_ANG_SPPED_ROLL = 5,
        
        // Make sure this number is 1 + the last action type.
        NUM_ACTION_TYPES = 6
    };

protected:
    RComAPI *rcom_;
    pthread_t readerThread_;
    int myID_;
    bool readerThreadIsUp_;

    static const struct timespec READER_SLEEPTIME_NSEC_;
    static const string EMPTY_STRING_;
    static const int SKIP_STATUSBAR_UPDATE_;

    void updateSensors_(void);
    void readerLoop_(void);
    void updateStatusBar_();

    static double getCurrentEpochTime_(void);
    static void *startReaderThread_(void *data);
    static void cbProcessRecievedData_(
        int recipientID,
        const void *data,
        size_t dataSize,
        bool reliableEnforced,
        void *extraArg);

public:
    RComHandler(RComHandler** a);
    ~RComHandler(void);
    bool initialize(int rcomAPIType);
    void run(void);
};

extern RComHandler *gRComHandler;
extern bool gRcomDebug;

#endif
/**********************************************************************
# $Log: rcom_handler.h,v $
# Revision 1.1  2008-07-15 22:33:52  endo
# OpenCV, Gazebo and USARSim added.
#
# Revision 1.3  2006/11/08 21:49:36  endo
# Regular backup.
#
# Revision 1.2  2006/11/07 17:02:26  endo
# RComHandler::cbProcessRecievedData_() implemented.
#
# Revision 1.1  2006/08/15 15:28:39  endo
# RComHandler implemented.
#
#**********************************************************************/
