/***********************************************************************

	sensor_blackboard.h

	Description:
		Define the sensor blackboard structure

	Modification History:
		Date		Author		Remarks
		10 NOV 99	Doug MacKenzie  Original


        Copyright 1999, Georgia Tech Research Corporation 
        Atlanta, Georgia  30332-0415
        ALL RIGHTS RESERVED, See file COPYRIGHT for details. 

*************************************************************************/

/* $Id: sensor_blackboard.h,v 1.4 2009-09-29 20:12:16 zkira Exp $ */

#include <vector>
#include "gt_message_types.h"
#include "gt_std_types.h"
#include "info_sharing.h"

using std::vector;

typedef struct 
{
   // The robot's X location in world coordinates.
   double x;

   // The robot's Y location in world coordinates.
   double y;
  
   // The robot's Z location in world coordinates.
   double z;

   // The robot's theta heading in world coordinates.
   double theta; // yaw

   // The robot's theta heading in world coordinates.
   double roll;

   // The robot's theta heading in world coordinates.
   double pitch;

   // Holds the latest obstacle readings.  
   // NOTE: Uses a C++ STL vector (variable size array) - NOT a AuRA Vector !!!!
   vector<obs_reading> sensed_objects;

   // Holds a list of all known objects (world map), uniquely identified by ID.  
   int num_world_objects;
   vector<info_sharing_object *> world_objects;

   // Holds a list of all known robots, uniquely identified by ID.  
   int num_world_robots;
   vector<info_sharing_robot *> world_robots;

} T_sensor_blackboard;

// Holds the latest copy of the sensor readings.
extern T_sensor_blackboard sensor_blackboard;

/**********************************************************************
 * $Log: sensor_blackboard.h,v $
 * Revision 1.4  2009-09-29 20:12:16  zkira
 * Information sharing code for MAST hardware demo
 * Added a database of all world objects that are known (not just immediately
 * sensed), that can be transmitted between robots
 *
 * Revision 1.3  2009/03/02 18:56:48  zkira
 * Full 3D information
 *
 * Revision 1.2  2008/09/09 20:12:33  zkira
 * Crude altitude control: Added z element (altitude) for 3D coordinates
 *
 * Revision 1.1.1.1  2008/07/14 16:44:18  endo
 * MAST Project (based on MissionLab-MINOS-20071018.tar.gz)
 *
 * Revision 1.1.1.1  2006/07/20 17:17:47  endo
 * MINOS Project (based on MissionLab-7.0.20060712.tar.gz)
 *
 * Revision 1.1.1.1  2006/07/12 13:37:56  endo
 * MissionLab 7.0
 *
 * Revision 1.1.1.1  2005/02/06 22:59:39  endo
 * AO-FNC Project (based on mlab-6.0.01.06192003.tar.gz)
 *
 * Revision 1.13  2003/04/06 13:20:20  endo
 * gcc 3.1.1
 *
 * Revision 1.12  2000/01/19 18:13:39  endo
 * *** empty log message ***
 *
 * Revision 1.11  2000/01/19 18:12:14  endo
 * Code checked in for Doug. He added blackboard variable
 * feature.
 *
 *
 **********************************************************************/
